#include "action.h"

ActionManager::ActionManager() {

    keys = 0; //NULL
    //How many joysticks to remember when plugged out
    joystick_memory_len = 2;

    map_key[BUT_A] = KEY_TYPE_ACT1;
    map_key[BUT_B] = KEY_TYPE_ACT2;
    map_key[BUT_C] = KEY_TYPE_ACT3;
    map_key[BUT_D] = KEY_TYPE_ACT4;

    map_key[BUT_LT] = KEY_TYPE_LTRIGGER;
    map_key[BUT_RT] = KEY_TYPE_RTRIGGER;

    map_key[BUT_START] = KEY_TYPE_START;
    map_key[BUT_SELECT] = KEY_TYPE_SELECT;

    map_key[BUT_LEFT] = KEY_TYPE_LEFT;
    map_key[BUT_RIGHT] = KEY_TYPE_RIGHT;
    map_key[BUT_UP] = KEY_TYPE_UP;
    map_key[BUT_DOWN] = KEY_TYPE_DOWN;

    // --~--

    map_joy[BUT_A] = JOY_TYPE_ACT1;
    map_joy[BUT_B] = JOY_TYPE_ACT2;
    map_joy[BUT_C] = JOY_TYPE_ACT3;
    map_joy[BUT_D] = JOY_TYPE_ACT4;

    map_joy[BUT_LT] = JOY_TYPE_LTRIGGER;
    map_joy[BUT_RT] = JOY_TYPE_RTRIGGER;

    map_joy[BUT_START] = JOY_TYPE_START;
    map_joy[BUT_SELECT] = JOY_TYPE_SELECT;

    map_joy[BUT_LEFT] = JOY_TYPE_LEFT;
    map_joy[BUT_RIGHT] = JOY_TYPE_RIGHT;
    map_joy[BUT_UP] = JOY_TYPE_UP;
    map_joy[BUT_DOWN] = JOY_TYPE_DOWN;

}

ActionManager::~ActionManager() {

    // Iterate and delete
    for( int i=0; i<list_joystick.size(); i++ ) {
		delete list_joystick[i];
	}
	for( int i=0; i<list_known_joysticks.size(); i++ ) {
		delete list_known_joysticks[i];
	}

}

void ActionManager::add_keyboard(Keyboard * inp_keyboard) {
    keys = inp_keyboard;
}

void ActionManager::add_joystick(Joystick * inp_joystick) {
    //Insert at beginning
    std::vector<Joystick*>::iterator it = list_joystick.begin();
    list_joystick.insert(it, inp_joystick);
}

void ActionManager::add_known_joystick(Joystick * inp_joystick) {
    //Insert at beginning
    std::vector<Joystick*>::iterator it = list_known_joysticks.begin();
    list_known_joysticks.insert(it, inp_joystick);
}


void ActionManager::config_joystick(int JoyNum) {

}


void ActionManager::manage_joysticks() {

    // Manage
    al_reconfigure_joysticks();

    // Remove unknown from list
    for( int m=0; m<list_joystick.size(); m++ ) {
        if(!al_get_joystick_active( list_joystick[m]->get_identity() )) {
            Joystick *unknown = list_joystick[m];
            unknown->inactive = true;
            add_joystick(unknown);
            list_joystick.erase(list_joystick.begin()+m);
        }
    }

    // Move known to list
    for( int m=0; m<list_known_joysticks.size(); m++ ) {
        if(al_get_joystick_active( list_known_joysticks[m]->get_identity() )) {
            Joystick *known = list_known_joysticks[m];
            known->inactive = false;
            add_joystick(known);
            list_known_joysticks.erase(list_known_joysticks.begin()+m);
        }
    }

    //Erase spare joysticks
    while(list_known_joysticks.size()>joystick_memory_len) {
        delete list_known_joysticks[list_known_joysticks.size()-1];
        list_known_joysticks.pop_back();
    }

    int old_len = list_joystick.size() + list_known_joysticks.size();
    int new_len = al_get_num_joysticks();

    int min_len = (new_len<=old_len)? new_len : old_len ;
    int max_len = (new_len>=old_len)? new_len : old_len ;


    //Add completely new ones
    for( int n=0; n<max_len-min_len; n++ ) {
        int jid = 0;
        ALLEGRO_JOYSTICK *j = al_get_joystick(jid);
        Joystick *k;
        jid++;

        bool foundOne = true;
        while( foundOne && (jid<list_joystick.size() || jid<list_known_joysticks.size()) ) {
            foundOne = false;
            for( int m=0; m<list_joystick.size() || m<list_known_joysticks.size(); m++ ) {
                if(m<list_joystick.size()) {
                    k = list_joystick[m];
                    if(k->get_identity()==j) {
                        foundOne = true;
                        j = al_get_joystick(jid);
                        jid++;
                    }
                }
                if(m<list_known_joysticks.size()) {
                    k = list_known_joysticks[m];
                    if(k->get_identity()==j) {
                        foundOne = true;
                        j = al_get_joystick(jid);
                        jid++;
                    }
                }
            }
        }

        Joystick *jstick = new Joystick(j);
        add_joystick(jstick);
        Joystick::start_msg("ADDED JOYSTICK");

    }

}


int ActionManager::get_num_joys() {
    return list_joystick.size();
}
Joystick * ActionManager::get_joy(int n) {
    return list_joystick[n];
}


bool ActionManager::held(ACTION_TYPE act) {

    if(act==BUT_ANY && keys->get_any()->down()) return true;
    else if(keys->get_key1(map_key[act])->down()
    || keys->get_key2(map_key[act])->down()) {
        return true;
    }

    int I = get_num_joys();
    for( int i=0; i<I; i++ ) {
        if(act==BUT_ANY && list_joystick[i]->get_any()->down()) return true;
        else if(list_joystick[i]->get_btn(map_joy[act])->down()) return true;
    }

    return false;

}

bool ActionManager::pressed(ACTION_TYPE act) {

    if(act==BUT_ANY && keys->get_any()->pressed()) return true;
    else if(keys->get_key1(map_key[act])->pressed()
    || keys->get_key2(map_key[act])->pressed()) {
        return true;
    }

    int I = get_num_joys();
    for( int i=0; i<I; i++ ) {
        if(act==BUT_ANY && list_joystick[i]->get_any()->pressed()) return true;
        else if(list_joystick[i]->get_btn(map_joy[act])->pressed()) return true;
    }

    return false;

}

bool ActionManager::released(ACTION_TYPE act) {

    if(act==BUT_ANY && keys->get_any()->released()) return true;
    else if(keys->get_key1(map_key[act])->released()
    || keys->get_key2(map_key[act])->released()) {
        return true;
    }

    int I = get_num_joys();
    for( int i=0; i<I; i++ ) {
        if(act==BUT_ANY && list_joystick[i]->get_any()->released()) return true;
        else if(list_joystick[i]->get_btn(map_joy[act])->released()) return true;
    }

    return false;

}


bool ActionManager::ANY() {
    if(keys->get_any()->pressed()) return true;
    for( int i=get_num_joys()-1; i>=0; i-- ) {
        if(list_joystick[i]->get_any()->pressed()) return true;
    }
    return false;
}
bool ActionManager::START() {
    if(keys->get_key1(map_key[BUT_START])->pressed()
    || keys->get_key2(map_key[BUT_START])->pressed()) return true;
    for( int i=get_num_joys()-1; i>=0; i-- ) {
        if(list_joystick[i]->get_btn(map_joy[BUT_START])->pressed()) return true;
    }
    return false;
}
bool ActionManager::SELECT() {
    if(keys->get_key1(map_key[BUT_SELECT])->pressed()
    || keys->get_key2(map_key[BUT_SELECT])->pressed()) return true;
    for( int i=get_num_joys()-1; i>=0; i-- ) {
        if(list_joystick[i]->get_btn(map_joy[BUT_SELECT])->pressed()) return true;
    }
    return false;
}
bool ActionManager::A() {
    if(keys->get_key1(map_key[BUT_A])->pressed()
    || keys->get_key2(map_key[BUT_A])->pressed()) return true;
    for( int i=get_num_joys()-1; i>=0; i-- ) {
        if(list_joystick[i]->get_btn(map_joy[BUT_A])->pressed()) return true;
    }
    return false;
}
bool ActionManager::B() {
    if(keys->get_key1(map_key[BUT_B])->pressed()
    || keys->get_key2(map_key[BUT_B])->pressed()) return true;
    for( int i=get_num_joys()-1; i>=0; i-- ) {
        if(list_joystick[i]->get_btn(map_joy[BUT_B])->pressed()) return true;
    }
    return false;
}
bool ActionManager::C() {
    if(keys->get_key1(map_key[BUT_C])->pressed()
    || keys->get_key2(map_key[BUT_C])->pressed()) return true;
    for( int i=get_num_joys()-1; i>=0; i-- ) {
        if(list_joystick[i]->get_btn(map_joy[BUT_C])->pressed()) return true;
    }
    return false;
}
bool ActionManager::D() {
    if(keys->get_key1(map_key[BUT_D])->pressed()
    || keys->get_key2(map_key[BUT_D])->pressed()) return true;
    for( int i=get_num_joys()-1; i>=0; i-- ) {
        if(list_joystick[i]->get_btn(map_joy[BUT_D])->pressed()) return true;
    }
    return false;
}
bool ActionManager::LT() {
    if(keys->get_key1(map_key[BUT_LT])->pressed()
    || keys->get_key2(map_key[BUT_LT])->pressed()) return true;
    for( int i=get_num_joys()-1; i>=0; i-- ) {
        if(list_joystick[i]->get_btn(map_joy[BUT_LT])->pressed()) return true;
    }
    return false;
}
bool ActionManager::RT() {
    if(keys->get_key1(map_key[BUT_RT])->pressed()
    || keys->get_key2(map_key[BUT_RT])->pressed()) return true;
    for( int i=get_num_joys()-1; i>=0; i-- ) {
        if(list_joystick[i]->get_btn(map_joy[BUT_RT])->pressed()) return true;
    }
    return false;
}
bool ActionManager::UP() {
    if(keys->get_key1(map_key[BUT_UP])->pressed()
    || keys->get_key2(map_key[BUT_UP])->pressed()) return true;
    for( int i=get_num_joys()-1; i>=0; i-- ) {
        if(list_joystick[i]->get_btn(map_joy[BUT_UP])->pressed()) return true;
    }
    return false;
}
bool ActionManager::DOWN() {
    if(keys->get_key1(map_key[BUT_DOWN])->pressed()
    || keys->get_key2(map_key[BUT_DOWN])->pressed()) return true;
    for( int i=get_num_joys()-1; i>=0; i-- ) {
        if(list_joystick[i]->get_btn(map_joy[BUT_DOWN])->pressed()) return true;
    }
    return false;
}
bool ActionManager::LEFT() {
    if(keys->get_key1(map_key[BUT_LEFT])->pressed()
    || keys->get_key2(map_key[BUT_LEFT])->pressed()) return true;
    for( int i=get_num_joys()-1; i>=0; i-- ) {
        if(list_joystick[i]->get_btn(map_joy[BUT_LEFT])->pressed()) return true;
    }
    return false;
}
bool ActionManager::RIGHT() {
    if(keys->get_key1(map_key[BUT_RIGHT])->pressed()
    || keys->get_key2(map_key[BUT_RIGHT])->pressed()) return true;
    for( int i=get_num_joys()-1; i>=0; i-- ) {
        if(list_joystick[i]->get_btn(map_joy[BUT_RIGHT])->pressed()) return true;
    }
    return false;
}


bool ActionManager::quit() {

    return (keys->get_key1(KEY_TYPE_QUIT)->pressed()
         || keys->get_key2(KEY_TYPE_QUIT)->pressed());

}
