#include "anim.h"


Sprite::Sprite(const char * filename) {

    path = filename;
    ResourceManager::resMan->add_resource(filename, FILE_TYPE_BMP, true);
	gfx = ResourceManager::resMan->get_bmp(filename);

	w = al_get_bitmap_width(gfx);
	h = al_get_bitmap_height(gfx);



}

Sprite::~Sprite() {

    ResourceManager::resMan->remove_resource(path, FILE_TYPE_BMP);

}

int Sprite::get_w() {

    return w;

}

int Sprite::get_h() {

    return h;

}

/*void Sprite::set(const char * filename) {

	ALLEGRO_BITMAP *bmp = al_load_bitmap(filename);

	if(!bmp) {
		bmp = al_create_bitmap(32,32);
		if(bmp) {

			ALLEGRO_BITMAP * temp = al_get_target_bitmap();
			al_set_target_bitmap(bmp);

			al_clear_to_color(al_map_rgb(255, 255, 0));

			al_set_target_bitmap(temp);

			gfx = bmp;
			isInitialized = true;
		}
		isLoaded = false;
	} else {
		gfx = bmp;
		isLoaded = true;
		isInitialized = true;
		al_convert_mask_to_alpha(gfx, al_map_rgb(255,0,255));
	}

	if(isInitialized) {
		w = al_get_bitmap_width(gfx);
		h = al_get_bitmap_height(gfx);
	}

}*/


ALLEGRO_BITMAP * Sprite::get() {

	return gfx;

}


void Sprite::draw(int x, int y) {

	al_draw_bitmap(gfx, x,y, 0);

}



Anim::Anim(const char * filename, int frames=1) : Sprite(filename) {
	num_frames = frames;
	index = 0;
	frame = 0;
	inc = 0;

	w/=num_frames;
}

void Anim::animate() {
    inc++;
    inc%=12;
    if(inc==0) frame++;
    frame%=num_frames;
}

void Anim::draw(int x, int y, int alp) {

	double dx = x-w/2.0;
	double dy = y-h/2.0;

	index = w*(frame%num_frames);
	al_draw_tinted_bitmap_region(gfx, al_map_rgba(255,255,255,alp), index, 0, w, h, dx,dy, 0);

}

void Anim::draw_ext(float x, float y, float ang, float alp, float xs, float ys) {

	float dx = x-w/2.0f;
	float dy = y-h/2.0f;

	al_draw_tinted_scaled_rotated_bitmap(gfx, al_map_rgba_f(1.0f,1.0f,1.0f,alp),  w/2.0f, h/2.0f, x,y, xs,ys,ang, 0);

}



Tileset::Tileset(const char * filename, int cols, int rows) : Sprite(filename) {
	num_cols = cols;
	num_rows = rows;
	num_cells = cols*rows;
	sep_x = 0;
	sep_y = 0;
	offset_x = 0;
	offset_y = 0;
	w = get_w()/cols;
	h = get_h()/rows;
}

void Tileset::draw(int cell, int x, int y) {

	double dx = offset_x + (cell % num_cols)*(w+sep_x);
	double dy = offset_y + (cell / num_cols)*(h+sep_y);
	double dw = w;
	double dh = h;

	al_draw_bitmap_region(gfx, dx,dy, dw,dh, x,y, 0);
}

void Tileset::draw_ext(int tx, int ty, int tw, int th, int x, int y) {

	double dx = tx;
	double dy = ty;
	double dw = tw;
	double dh = th;

	al_draw_bitmap_region(gfx, dx,dy, dw,dh, x,y, 0);
}

int Tileset::get_index(int col, int row) {
	return (row * num_cols)+col;
}

void Tileset::settings(int width, int height, int xoff, int yoff, int sepx, int sepy) {

	sep_x = sepx;
	sep_y = sepy;
	offset_x = xoff;
	offset_y = yoff;
	w = width;
	h = height;

}



Strip::Strip(const char * filename, int cells) : Sprite(filename) {
	num_cells = cells;
	w = get_w()/cells;
}

void Strip::draw(int cell, int x, int y, bool flipx, bool flipy) {

	double dx = cell*w;
	double dy = 0;

    int flags = 0;
    if(flipx && flipy) flags = ALLEGRO_FLIP_HORIZONTAL | ALLEGRO_FLIP_VERTICAL;
    else if(flipx) flags = ALLEGRO_FLIP_HORIZONTAL;
    else if(flipy) flags = ALLEGRO_FLIP_VERTICAL;

	al_draw_bitmap_region(gfx, dx,dy, w,h, x,y, flags);
}
