#include "application.h"



Application::Application(ALLEGRO_DISPLAY *scr_disp, Settings *configuration) :
	inp_keyboard(),
	inp_mouse(),
	act(),
	scale(),
	trans(configuration->get_screen_w(), configuration->get_screen_h()) {

    disp = scr_disp;
    config = configuration;

    ResourceManager::resMan = new ResourceManager();

	display_width = configuration->get_screen_w();
	display_height = configuration->get_screen_h();

    partsys = new IPartSystem();
    ResourceManager::resMan->add_resource("fnt/DAYROM__.ttf", FILE_TYPE_FNT, true, 14);
	fnt = ResourceManager::resMan->get_fnt("fnt/DAYROM__.ttf", 14);

	RootMenu::app = this;
	GamesHandler::app = this;
	//Game::app = this;

    act.add_keyboard(&inp_keyboard);

	menu = 0;

	games = new GamesHandler(config);

	state = APP_STATE_GAME_INIT;
	new_menu = APP_MENU_ID_MAIN;
	//set_menu(APP_MENU_ID_MAIN);

	bmp = al_create_bitmap(display_width, display_height);

	if(!bmp) {
		fprintf(stderr, "create_bitmap failed!\n");
	}

	// Draw application BG -->>
	ALLEGRO_BITMAP * backbuf_tmp = al_get_target_bitmap();
	al_set_target_bitmap(bmp);

	al_clear_to_color(al_map_rgb(255,255,255));

	al_set_target_bitmap(backbuf_tmp);
	// <<--

    // Standard dialog
    in_dialog = false;
    dialog_id = 0;
    dialog = 0;
    // --

	redraw = true;
	doexit = false;
	anim_time = 0;

	//gameInProgress = new Game();
    act.manage_joysticks();
    al_identity_transform(&scale);
    scale_xpos = 0;
    scale_ypos = 0;
    scale_factor = 1;
}

Application::~Application() {

    if(dialog!=0) delete dialog;

    games -> end_game();

	delete games;


	al_destroy_bitmap(bmp);

    // Not stricly necessary
	ResourceManager::resMan->remove_resource("fnt/DAYROM__.ttf", FILE_TYPE_FNT);

	delete menu;
	delete partsys;

}

void Application::fullscreen(bool is_fullscreen) {

    al_toggle_display_flag(disp, ALLEGRO_FULLSCREEN_WINDOW, is_fullscreen);

    scale_factor = 1;
    while(al_get_display_width(disp)>=display_width*(scale_factor+1)
    && al_get_display_height(disp)>=display_height*(scale_factor+1)) {

        scale_factor += 1;

    }
    scale_xpos = (al_get_display_width(disp)-display_width*scale_factor)/(scale_factor*2);
    scale_ypos = (al_get_display_height(disp)-display_height*scale_factor)/(scale_factor*2);

    // Apply transformation scale to screen bitmap
    al_identity_transform(&scale);
    al_scale_transform(&scale, scale_factor, scale_factor);
    al_use_transform(&scale);

}

void Application::resize(int width, int height) {

    display_width = width;
    display_height = height;

    al_resize_display(disp, width, height);
    al_destroy_bitmap(bmp);
    bmp = al_create_bitmap(width, height);

    if(!bmp) {
        fprintf(stderr, "create_bitmap failed!\n");
    }

    // Draw application BG -->>
    ALLEGRO_BITMAP * backbuf_tmp = al_get_target_bitmap();
    al_set_target_bitmap(bmp);

    al_clear_to_color(al_map_rgb(255,255,255));

    al_set_target_bitmap(backbuf_tmp);
    // <<--

    scale_factor = 1;
    while(al_get_display_width(disp)>=display_width*(scale_factor+1)
    && al_get_display_height(disp)>=display_height*(scale_factor+1)) {

        scale_factor += 1;

    }
    scale_xpos = (al_get_display_width(disp)-display_width*scale_factor)/(scale_factor*2);
    scale_ypos = (al_get_display_height(disp)-display_height*scale_factor)/(scale_factor*2);

    // Apply transformation scale to screen bitmap
    al_identity_transform(&scale);
    al_scale_transform(&scale, scale_factor, scale_factor);
    al_use_transform(&scale);


}

void Application::set_app_state(APP_STATE new_state, bool do_transition) {
    if(do_transition) trans.setup(bmp, 16, TRANS_TYPE_RANDOM);
    else trans.setup(bmp, 0, TRANS_TYPE_RANDOM);
    state = new_state;
}

void Application::set_menu(APP_MENU_ID menu2open) {

    new_menu = menu2open;
    set_app_state(APP_STATE_MENU_INIT, false);

}

void Application::create_menu() {

    delete menu;

    switch(new_menu) {
        case APP_MENU_ID_OPTKEYS:
            menu = new OptionsKeyboardMenu(partsys, inp_keyboard);
        break;
        case APP_MENU_ID_OPTJOYS:
            menu = new OptionsJoystickMenu(partsys, act);
        break;
        case APP_MENU_ID_MAIN:
        default:
            menu = new MainMenu(partsys, act);
        break;
    }
}

void Application::update(ALLEGRO_EVENT * event) {

	if(doexit) return;

	if(event->type == ALLEGRO_EVENT_TIMER) {

		// Trigger delayed input keeping events in sync
		int I = act.get_num_joys();
		for( int i=0; i<I; i++ ) act.get_joy(i)->on_timer();
		inp_keyboard.on_timer();
		inp_mouse.on_timer();

        // For animated sprites etc. :-S
        anim_time++;
        // For particles
        partsys->update();

        // Advance a frame
        if(in_dialog) {

            // Prompt action
            switch(dialog_id) {
                case APP_DIALOG_ID_BEFOREQUIT:
                case APP_DIALOG_ID_BEFOREMENU:
                default:
                    dialog->update(act, inp_keyboard);
                break;
            }

            if(dialog->done()) {
                // Player choose yes
                switch(dialog_id) {
                    case APP_DIALOG_ID_BEFOREQUIT:
                        if(dialog->get_choice()==DIALOGMENU_BUTTON_ID_OK) {
                            doexit = true;
                        }
                    break;
                }
            }

            if(dialog->done()) {
                delete dialog;
                dialog = 0;
                dialog_id = 0;
                in_dialog = false;
            }

        } else {


            switch(state) {
                case APP_STATE_GAME_INIT:

                    delete menu;
                    menu = new MainMenu(partsys, act);

                    //CREATE A NEW GAME
                    games->new_game();
                    set_app_state(APP_STATE_GAME, true);

                case APP_STATE_GAME:

                    games->get_game()->run(anim_time, act, inp_keyboard, inp_mouse);

                    if(act.SELECT()) {

                        //games->end_game();
                        set_menu(APP_MENU_ID_MAIN);

                    }

                    if(act.quit()) {

                        if(!in_dialog) {
                            dialog_id = APP_DIALOG_ID_BEFOREQUIT;
                            in_dialog = true;
                            dialog = new DialogMenu(partsys, "Quit game?", true);
                        }
                    }

                break;
                case APP_STATE_MENU_INIT:

                    create_menu();
                    set_app_state(APP_STATE_MENU, false);

                case APP_STATE_MENU:


                    menu->update(act, inp_keyboard);

                break;
                case APP_STATE_QUIT:
                default:

                    if(!trans.isTransfering()) {
                        doexit = true;
                    }

                break;
            }

            if(trans.isTransfering()) {
                trans.transfer(bmp);
            }

        }

        //QUICKTOGGLE FULLSCREEN MODE
        if(inp_keyboard.get_any()->pressed()) {
            if(inp_keyboard.get_anycode()==ALLEGRO_KEY_F4) {
                config -> set_fullscreen(true);
                fullscreen(true);
            }
            if(inp_keyboard.get_anycode()==ALLEGRO_KEY_F3) {
                config -> set_fullscreen(false);
                fullscreen(false);
            }
        }



		redraw = true;
	}
	else if(event->type == ALLEGRO_EVENT_DISPLAY_CLOSE) {

	    if(state==APP_STATE_GAME) {

	        if(!in_dialog) {
	            dialog_id = APP_DIALOG_ID_BEFOREQUIT;
                in_dialog = true;
                dialog = new DialogMenu(partsys, "Quit without saving?", true);
	        }

	    } else doexit = true;
	}
	else if(event->type == ALLEGRO_EVENT_DISPLAY_SWITCH_OUT) {
		///TODO: SHOULDN'T THIS BE SWITCH_OUT??? (still testing)
		// Lost focus constitutes as a control release
		int I = act.get_num_joys();
		for( int i=0; i<I; i++ ) act.get_joy(i)->on_blur();
		inp_keyboard.on_blur();
		inp_mouse.on_blur();

	}
	else if(event->type == ALLEGRO_EVENT_DISPLAY_RESIZE) {
		/*
		// Fix
		al_acknowledge_resize(disp);

        ALLEGRO_TRANSFORM transform;
        al_identity_transform(&transform);
        al_scale_transform(&transform, al_get_display_width(disp)/display_width, al_get_display_height(disp)/display_height);
        al_use_transform(&transform);
        */

	}
	else if(event->type == ALLEGRO_EVENT_JOYSTICK_CONFIGURATION) {
		// Call the appropriate function and show what happened on the screen
		act.manage_joysticks();
        Joystick::start_msg("Joystick plugin/out detected.");
	}
	else {

		//Pass on the event to those interested
		int I = act.get_num_joys();
		for( int i=0; i<I; i++ ) act.get_joy(i)->on_event(event);
		inp_keyboard.on_event(event);
		inp_mouse.on_event(event, -scale_xpos, -scale_ypos, scale_factor);

	}
}

void Application::draw() {

	if(doexit) return;

	ALLEGRO_BITMAP * backbuf_tmp = al_get_target_bitmap();
    al_set_target_bitmap(bmp);

	switch(state) {
		case APP_STATE_GAME:

			//Usually just draws all views, and the hud on top
			games->get_game()->display(anim_time);

		break;
		case APP_STATE_MENU:

			menu->draw(anim_time);

		break;
		case APP_STATE_QUIT:

            al_clear_to_color(al_map_rgb(0,0,0));
			al_draw_line(0,0,display_width,display_height,al_map_rgb(255,255,255), 0);
			al_draw_line(0,display_height,display_width,0,al_map_rgb(255,255,255), 0);

		break;
	}

    al_set_target_bitmap(backbuf_tmp);
    al_clear_to_color(al_map_rgb(0,0,0));

    if(config->get_fullscreen()) {

        if(trans.isTransfering()) al_draw_bitmap(trans.get(), scale_xpos, scale_ypos, 0);
        else al_draw_bitmap(bmp, scale_xpos, scale_ypos, 0);



    } else {
        if(trans.isTransfering()) al_draw_bitmap(trans.get(), 0,0, 0);
        else al_draw_bitmap(bmp, 0,0, 0);
    }



    if(in_dialog) {
        dialog->draw(anim_time);
    }

    Joystick::display_msg(fnt);


	redraw = false;

}
