#ifndef APPLICATION_H_INCLUDED
#define APPLICATION_H_INCLUDED

class GamesHandler;
class RootMenu;
class DialogMenu;
class MainMenu;
class NewGameMenu;
class OptionsMenu;
class OptionsGraphicsMenu;
class OptionsJoystickMenu;
class OptionsKeyboardMenu;

#include "configure.h"
#include "resources.h"
#include "menu.h"
#include "DialogMenu.h"
#include "mainMenu.h"
#include "OptionsJoystickMenu.h"
#include "OptionsKeyboardMenu.h"
#include "gamesHandler.h"
#include "transition.h"
#include "key.h"
#include "joystick.h"
#include "mouse.h"
#include "interactive.h"


enum APP_STATE {
	APP_STATE_QUIT,
	APP_STATE_GAME_INIT,
	APP_STATE_GAME,
	APP_STATE_MENU_INIT,
	APP_STATE_MENU
};

enum {
    APP_DIALOG_ID_BEFOREQUIT = 1,
    APP_DIALOG_ID_BEFOREMENU
};

enum APP_MENU_ID {
    APP_MENU_ID_MAIN = 1,
    APP_MENU_ID_NEWGAME,
    APP_MENU_ID_OPTIONS,
    APP_MENU_ID_OPTKEYS,
    APP_MENU_ID_OPTJOYS,
    APP_MENU_ID_OPTGFXS
};

class Application {

	public:

	Application(ALLEGRO_DISPLAY *scr_disp, Settings *configuration);
	~Application();

	void update(ALLEGRO_EVENT * event);
	void draw();

	bool redraw;
	bool doexit;
	long anim_time; // DO NOT USE !

    void set_app_state(APP_STATE new_state, bool do_transition);
    void set_menu(APP_MENU_ID menu2open);

    void fullscreen(bool is_fullscreen);
    void resize(int width, int height);

    IPartSystem *partsys;
    Settings *config;

	private:


	APP_STATE state;
	int display_width;
	int display_height;

	ALLEGRO_DISPLAY *disp;
	ALLEGRO_BITMAP *bmp;

    ALLEGRO_TRANSFORM scale;
    int scale_xpos;
    int scale_ypos;
    int scale_factor;

    bool in_dialog;
    int dialog_id;
    DialogMenu *dialog;

	GamesHandler *games;

	Keyboard inp_keyboard;
	Mouse inp_mouse;
	ActionManager act;

    Transition trans;

	// Interactive menus:
	RootMenu *menu;

    APP_MENU_ID new_menu;
	void create_menu();

	ALLEGRO_FONT *fnt;
};

#endif // APPLICATION_H_INCLUDED
