#include <iostream>
#include "interactive.h"

Interactive *IPartSystem::topInteract = 0;

IPartSystem::IPartSystem() {

	part_num = 0;
	part_max = 200;
	for( int p=0; p<part_max; p++ ) {
	    particles[p].x = 0.0;
	    particles[p].y = 0.0;
	    particles[p].xsp = 0.25 + lengthdir_x(0.1, rand()%360);
	    particles[p].ysp = 0.25 + lengthdir_x(0.1, rand()%360);
	}

    part_count = 0;
	part_delay = 50;

	part_anim_count = 0;
	part_anim_delay = 60;

	bmp = al_create_bitmap(32,32);
	ALLEGRO_BITMAP * backbuf_tmp = al_get_target_bitmap();
	al_set_target_bitmap(bmp);

	al_clear_to_color(al_map_rgba_f(1.0,1.0,1.0,0.0));
	for( int y=0; y<32; y++ ) {
	    for( int x=0; x<32; x++ ) {

            double dist = sqrt((x-16)*(x-16) + (y-16)*(y-16));
            if(dist<=16.0) {
                al_draw_pixel(x,y, al_map_rgba_f(1.0,1.0,1.0,1.0-dist/16.0));
            }

        }
	}

    partn = 0;

	al_set_target_bitmap(backbuf_tmp);

}

ALLEGRO_BITMAP * IPartSystem::get_bmp() {
    return bmp;
}


double IPartSystem::get_part_x(int index) {
    return particles[index].x;
}

double IPartSystem::get_part_y(int index) {
    return particles[index].y;
}

double IPartSystem::get_part_xsp(int index) {
    return particles[index].xsp;
}

double IPartSystem::get_part_ysp(int index) {
    return particles[index].ysp;
}

void IPartSystem::set_part(int index, double p_x, double p_y, double p_xsp, double p_ysp) {

    particles[index].xsp += (p_x-particles[index].x)*0.1;
    particles[index].ysp += (p_y-particles[index].y)*0.1;
    particles[index].x += particles[index].xsp+(rand()%3-1)*0.5;
    particles[index].y += particles[index].ysp+(rand()%3-1)*0.5;
    particles[index].xsp *= 0.8;
    particles[index].ysp *= 0.8;


}

double IPartSystem::get_part_time() {
    return double(part_anim_count); // /double(part_anim_delay);
}

int IPartSystem::get_part_count() {
    return part_num;
}

void IPartSystem::part_reset() {
    part_count = 0;
}

void IPartSystem::update() {



    // Particles
    part_anim_count = (part_anim_count+1); //%part_anim_delay;
    for( int i=0; i<15; i++ ) {
        if(part_num<part_max) {
            particles[part_num].x = rand()%640;
            particles[part_num].y = rand()%480;
            part_num++;
        }
    }

    // --

}

// --~--


IButton::IButton(Interactive *con, IPartSystem *sys, int nx, int ny, int nw, int nh, const char *nl, int index) {
	x = nx;
	y = ny;
	w = nw;
	h = nh;
	label = nl;

	adj_up = 0;
	adj_left = 0;
	adj_right = 0;
	adj_down = 0;

    is_disabled = false;

	has_focus = true;
	container = con;
	partsys = sys;

	id = index;

}

void IButton::disable() {
    is_disabled = true;
}

int IButton::get_x() {
    return x;
}

int IButton::get_y() {
    return y;
}

int IButton::get_w() {
    return w;
}

int IButton::get_h() {
    return h;
}

void IButton::set_x(int nx) {
    x = nx;
}

void IButton::set_y(int ny) {
    y = ny;
}

void IButton::set_label(const char *newlabel) {
    label = newlabel;
}

int IButton::use(ActionManager& act, Keyboard& keys) {

	if(!has_focus) return 0;

	bool up = act.UP();
	bool left = act.LEFT();
	bool right = act.RIGHT();
	bool down = act.DOWN();

	if(act.quit()) { return 0; }
	else if(up) container->set_used_but(adj_up);
	else if(left) container->set_used_but(adj_left);
	else if(right) container->set_used_but(adj_right);
	else if(down) container->set_used_but(adj_down);
	else if(act.held()) {
		return id;
	}

	return 0;

}

void IButton::update() {

    if(!has_focus || partsys->topInteract!=container) return;

	// Particle control
	double t = partsys->get_part_time();
	int I = partsys->get_part_count();
	for( int k=0; k<I; k++ ) {

        int i = rand()%I;

        double xx,yy;
        double j = ((double) ((i+int(t))%I))/I;
        //j+=t/1.0; // 0.2 rounds per second
        while(j>1.0) j-=1.0;
        while(j<0.0) j+=1.0;
        ol_to_coords(this, j, xx, yy);
        xx+=x;
        yy+=y;
        partsys->set_part(i, xx,yy, 0,0);

    }
    // --

}

void IButton::draw(int offx, int offy, ALLEGRO_FONT *fnt, int t) {

	int ox = offx+x;
	int oy = offy+y;

    /*
    if(adj_up!=0) al_draw_line(ox+w/2,oy+h/2,offx+adj_up->x+adj_up->w/2,offy+adj_up->y+adj_up->h/2,al_map_rgba(200,200,200,255), 0.0);
    if(adj_left!=0) al_draw_line(ox+w/2,oy+h/2,offx+adj_left->x+adj_left->w/2,offy+adj_left->y+adj_left->h/2,al_map_rgba(200,200,200,255), 0.0);
    if(adj_right!=0) al_draw_line(ox+w/2,oy+h/2,offx+adj_right->x+adj_right->w/2,offy+adj_right->y+adj_right->h/2,al_map_rgba(200,200,200,255), 0.0);
    if(adj_down!=0) al_draw_line(ox+w/2,oy+h/2,offx+adj_down->x+adj_down->w/2,offy+adj_down->y+adj_down->h/2,al_map_rgba(200,200,200,255), 0.0);
    */

    ALLEGRO_COLOR txt_color;
    if(is_disabled) txt_color = al_map_rgb(180,180,180);
    else txt_color = al_map_rgb(0,0,0);

	al_draw_filled_rectangle(ox,oy,ox+w,oy+h,al_map_rgba(200,0,55,120));
	al_draw_rectangle(ox,oy,ox+w,oy+h,al_map_rgba(255,255,255,100), 0.0);



	if(has_focus && partsys->topInteract==container) {
	    al_set_blender(ALLEGRO_DEST_MINUS_SRC, ALLEGRO_INVERSE_ALPHA, ALLEGRO_ONE);
	    al_set_blender(ALLEGRO_SRC_MINUS_DEST, ALLEGRO_ALPHA, ALLEGRO_INVERSE_ALPHA);
	    al_set_blender(ALLEGRO_ADD, ALLEGRO_ALPHA, ALLEGRO_ONE);

	    int I = partsys->get_part_count();
		for( int i=0; i<I; i++ ) {

			double tx,ty, nx,ny;
			tx = ox + partsys->get_part_x(i) - x;
			ty = oy + partsys->get_part_y(i) - y;
			nx = ox + partsys->get_part_x((i+1)%I) - x;
			ny = oy + partsys->get_part_y((i+1)%I) - y;

            double J;
            for( int j=0; j<8; j++ ) {
                J = double(j)/5.0;
                if(i%2==0) al_draw_tinted_scaled_bitmap(partsys->get_bmp(), al_map_rgba(255,255,255, 20), 0,0,32,32, tx*(1.0-J)+nx*J+lengthdir_x(20, i*1440/I),ty*(1.0-J)+ny*J+lengthdir_y(20, i*1440/I), abs(j-3)*2, abs(j-3)*2, 0);
                else if(i%1==0) al_draw_tinted_scaled_bitmap(partsys->get_bmp(), al_map_rgba(255,255,255, 20), 0,0,32,32, tx*(1.0-J)+nx*J+lengthdir_x(20, i*1440/I),ty*(1.0-J)+ny*J+lengthdir_y(20, i*1440/I), abs(j-3)*2, abs(j-3)*2, 0);
                //else al_draw_tinted_scaled_bitmap(partsys->get_bmp(), al_map_rgba(0,127,0, 25), 0,0,32,32, tx*(1.0-J)+nx*J,ty*(1.0-J)+ny*J,10,10,0);
                //else al_draw_bitmap(partsys->get_bmp(), tx*(1.0-J)+nx*J,ty*(1.0-J)+ny*J,abs(j-3)*2, al_map_rgba(255-0,255-127,255-0, 255-25));
            }


		}

		al_set_blender(ALLEGRO_ADD, ALLEGRO_ALPHA, ALLEGRO_INVERSE_ALPHA);
	}

    al_draw_text(fnt, txt_color, ox+8-4, oy+8-4, 0, label);

}

void IButton::pass_focus(IButton *target) {

	blur();
	target->focus();

}

void IButton::setup_adj(IButton *up, IButton *left, IButton *right, IButton *down) {

	adj_up = up;
	adj_left = left;
	adj_right = right;
	adj_down = down;

}

void IButton::focus() {


	has_focus = true;

}

void IButton::blur() {

	has_focus = false;

}

void IButton::ol_to_coords(IButton *subject, double frac, double& x, double& y) {

	double r = CORNER_RADIUS;

	double nw, ne, sw, se;
	nw = ne = sw = se = ALLEGRO_PI*r/2.0;

	double n, w, e, s;
	n = s = subject->w-r*2.0;
	w = e = subject->h-r*2.0;

	double tot = n+ne+e+se+s+sw+w+nw;
	double result = frac*tot;

	if(result<=n) {
		x = r+result;
		y = 0.0;
		return;
	}
	result-=n;

	if(result<=ne) {
		double angle = 90.0-result*90.0/ne;
		x = r+n+lengthdir_x(r, angle);
		y = r+lengthdir_y(r, angle);
		return;
	}
	result-=ne;

	if(result<=e) {
		x = n+r*2.0;
		y = r+result;
		return;
	}
	result-=e;

	if(result<=se) {
		double angle = 360.0-result*90.0/se;
		x = r+n+lengthdir_x(r, angle);
		y = r+e+lengthdir_y(r, angle);
		return;
	}
	result-=se;

	if(result<=s) {
		x = r+s-result;
		y = r*2.0+w;
		return;
	}
	result-=s;

	if(result<=sw) {
		double angle = 270.0-result*90.0/sw;
		x = r+lengthdir_x(r, angle);
		y = r+w+lengthdir_y(r, angle);
		return;
	}
	result-=sw;

	if(result<=w) {
		x = 0.0;
		y = r+w-result;
		return;
	}
	result-=w;

	if(result<=nw) {
		double angle = 180.0-result*90.0/nw;
		x = r+lengthdir_x(r, angle);
		y = r+lengthdir_y(r, angle);
		return;
	}
	result-=nw;

	//Safety net
	x=r;
	y=0;

}


// --~--


IGetVal::IGetVal(Interactive *con, IPartSystem *sys, int nx, int ny, int nw, int nh, int startv, int minv, int maxv, int index) :
    IButton(con, sys, nx, ny, nw, nh, "", index) {

    minval = minv;
    maxval = maxv;
    value = startv;

    submit_id = 0;
    cancel_id = 0;

    if(value<minval) value=minval;
    if(value>maxval) value=maxval;

}

int IGetVal::get_value() {
    return value;
}

void IGetVal::set_value(int newval) {
    value = newval;
}

void IGetVal::set_submitCancel(int submit, int cancel) {
    submit_id = submit;
    cancel_id = cancel;
}

int IGetVal::use(ActionManager& act, Keyboard& keys) {

    if(!has_focus) return 0;

	bool up = act.UP();
	bool down = act.DOWN();

	if(act.quit()) { return 0; }
	else if(up) container->set_used_but(adj_up);
	else if(down) container->set_used_but(adj_down);
	else if(act.LT()
         || act.A()
         || act.B()
         || act.LEFT()) {
	    value--;
	    if(value<minval) value = minval;
	    return id;
	}
	else if(act.RT()
         || act.C()
         || act.D()
         || act.RIGHT()) {
	    value++;
	    if(value>maxval) value = maxval;
	    return id;
	}
	else if(act.START()) {
		return submit_id;
	}
	else if(act.SELECT()) {
		return cancel_id;
	}
	else if(act.held()) {
		return id;
	}

	return 0;


}

void IGetVal::update() {

    if(!has_focus || partsys->topInteract!=container) return;

	// Particle control
	double t = partsys->get_part_time();
	int I = partsys->get_part_count();
	double tx, ty;
	double px, py;
        px = partsys->get_part_x(I-1);
        py = partsys->get_part_y(I-1);
    for( int i=0; i<I; i++ ) {

        tx = partsys->get_part_x(i)-x;
        ty = partsys->get_part_y(i)-y;
        double xx,yy;
        double j = ((double) i)/I;
        j+=t/1.0; // 1.0 rounds per second
        while(j>1.0) j-=1.0;
        while(j<0.0) j+=1.0;
        ol_to_coords(this, j, xx, yy);
        xx+=x;
        yy+=y;
        //xx += (xx - tx)/12.0;
        //yy += (yy - ty)/12.0;
        partsys->set_part(i, xx,yy, 0,0);
        px = tx;
        py = ty;

    }
    // --

}

void IGetVal::draw(int offx, int offy, ALLEGRO_FONT *fnt, int t) {

	int ox = offx+x;
	int oy = offy+y;

    al_draw_filled_rectangle(ox,oy,ox+w,oy+h,al_map_rgba(100,159,255,100));

	al_draw_rectangle(ox,oy,ox+15,oy+h,al_map_rgba(155,155,55,100), 0.0);
	al_draw_text(fnt, al_map_rgb(0,0,0), ox+8-4, oy+8-4, 0, "-");
	al_draw_rectangle(ox+15,oy,ox+w-15,oy+h,al_map_rgba(155,155,55,100), 0.0);
	al_draw_textf(fnt, al_map_rgb(0,0,0), ox+15+8, oy+8-4, 0, "%d", value);
	al_draw_rectangle(ox+w-15,oy,ox+w,oy+h,al_map_rgba(155,155,55,100), 0.0);
	al_draw_text(fnt, al_map_rgb(0,0,0), ox+w-15+8-4, oy+8-4, 0, "+");

	if(has_focus && partsys->topInteract==container) {
        //al_set_blender(ALLEGRO_DEST_MINUS_SRC, ALLEGRO_INVERSE_ALPHA, ALLEGRO_ONE);
	    int I = partsys->get_part_count();
		for( int i=0; i<I; i++ ) {
			double xx,yy;
			xx = partsys->get_part_x(i);
			yy = partsys->get_part_y(i);
			if(i%4==0) al_draw_filled_circle(ox-x+xx+lengthdir_x(10, i*1440/I),oy-y+yy+lengthdir_y(10, i*1440/I),10, al_map_rgba(0,127,0, 25));
			else if(i%2==0) al_draw_filled_circle(ox-x+xx+lengthdir_x(5, i*1440/I),oy-y+yy+lengthdir_y(5, i*1440/I),10, al_map_rgba(0,127,127, 25));
			else al_draw_filled_circle(ox-x+xx,oy-y+yy,10, al_map_rgba(0,255,0, 6));
		}
		//al_set_blender(ALLEGRO_ADD, ALLEGRO_ALPHA, ALLEGRO_INVERSE_ALPHA);
	}


}


// --~--


IGetKey::IGetKey(Interactive *con, IPartSystem *sys, int nx, int ny, int nw, int nh, int k1, int k2, const char *nl, int index) :
    IButton(con, sys, nx, ny, nw, nh, "", index) {

    label = nl;

    value1 = k1;
    value2 = k2;

    conflict1 = conflict2 = false;

    in_state = false;

    submit_id = 0;
    cancel_id = 0;

}

int IGetKey::get_value1() {
    return value1;
}
int IGetKey::get_value2() {
    return value2;
}

void IGetKey::set_values(int newval1, int newval2) {
    value1 = newval1;
    value2 = newval2;
}

void IGetKey::set_submitCancel(int submit, int cancel) {
    submit_id = submit;
    cancel_id = cancel;
}

int IGetKey::use(ActionManager& act, Keyboard& keys) {

    if(!has_focus) return 0;

    if(in_state) {

        bool up = keys.get_key1(KEY_TYPE_UP)->pressed();
        bool left = keys.get_key1(KEY_TYPE_LEFT)->pressed();
        bool right = keys.get_key1(KEY_TYPE_RIGHT)->pressed();
        bool down = keys.get_key1(KEY_TYPE_DOWN)->pressed();

        if(up || down || left || right
        || act.quit()) {

            in_state = false;
            return 0;

        }

        if(keys.get_any()->pressed()) {
            if(is_disabled) {
                value2 = keys.get_anycode();
            } else {
                int val = keys.get_anycode();
                if(val==value1) value2 = val;

                value1 = value2;
                value2 = val;
            }

            in_state = false;
            return id;
        }


    } else {

        if(act.quit()) { return 0; }
        else if(act.UP()) container->set_used_but(adj_up);
        else if(act.DOWN()) container->set_used_but(adj_down);
        else if(act.LEFT()) container->set_used_but(adj_left);
        else if(act.RIGHT()) container->set_used_but(adj_right);
        else if(act.SELECT()) {
            return 0;
        }
        else if(act.ANY()) {

            in_state = true;
        }

    }

	return 0;


}

KEY_TYPE IGetKey::test(Keyboard& keys) {
    conflict1 = conflict2 = false;
    KEY_TYPE k1 = keys.conflict(value1);
    KEY_TYPE k2 = keys.conflict(value2);
    if(k1!=KEY_TYPE_QUIT) conflict1 = true;
    if(k2!=KEY_TYPE_QUIT) conflict2 = true;
    if(conflict1) return k1;
    else return k2;
}

void IGetKey::update() {

    if(!has_focus || partsys->topInteract!=container) return;

	// Particle control
	double t = partsys->get_part_time();
	int I = partsys->get_part_count();
    for( int i=0; i<I; i++ ) {

        double xx,yy;
        double j = ((double) i)/I;
        j+=t/1.0; // 1.0 rounds per second
        while(j>1.0) j-=1.0;
        while(j<0.0) j+=1.0;
        ol_to_coords(this, j, xx, yy);
        xx+=x;
        yy+=y;
        if(in_state) xx += w+3;
        partsys->set_part(i, xx,yy, 0,0);

    }
    // --

}

void IGetKey::draw(int offx, int offy, ALLEGRO_FONT *fnt, int t) {

	int ox = offx+x;
	int oy = offy+y;

    al_draw_filled_rectangle(ox,oy,ox+w,oy+h,al_map_rgba(100,159,255,100));
    al_draw_filled_rectangle(ox+w+3,oy,ox+w+4+w,oy+h,al_map_rgba(0,0,0,100));

	al_draw_rectangle(ox,oy,ox+w,oy+h,al_map_rgba(155,155,55,100), 0.0);
	al_draw_rectangle(ox+w+3,oy,ox+w+4+w,oy+h,al_map_rgba(155,155,55,100), 0.0);


    ALLEGRO_COLOR txt_color, key_color;
    if(is_disabled) txt_color = al_map_rgb(0,0,0);
    else txt_color = al_map_rgb(0,0,0);

    if(conflict1 || conflict2) key_color = al_map_rgb(255,0,0);
    else key_color = al_map_rgb(0,255,0);

    al_draw_text(fnt, txt_color, ox+8-4, oy+8-4, 0, label);
    if(in_state) {
        al_draw_text(fnt, key_color, ox+8+4+3+w, oy+8-4, 0, "<Press any key>");
    } else if(value1==value2) {
        if(conflict1 || conflict2) al_draw_textf(fnt, al_map_rgb(255,0,0), ox+8+4+3+w, oy+8-4, 0, "%s", al_keycode_to_name(value1));
        else al_draw_textf(fnt, al_map_rgb(0,255,0), ox+8+4+3+w, oy+8-4, 0, "%s", al_keycode_to_name(value1));
        if(is_disabled) al_draw_textf(fnt, al_map_rgba(0,0,0, 195), ox+8+4+3+w, oy+8-4, 0, "%s", al_keycode_to_name(value1));
    } else {
        if(conflict1 && conflict2) {
            al_draw_textf(fnt, al_map_rgb(255,0,0), ox+8+4+3+w, oy+8-4, 0, "%s, %s", al_keycode_to_name(value1), al_keycode_to_name(value2));
            if(is_disabled) al_draw_textf(fnt, al_map_rgba(0,0,0, 195), ox+8+4+3+w, oy+8-4, 0, "%s", al_keycode_to_name(value1));
        } else if(conflict1) {
            al_draw_textf(fnt, al_map_rgb(0,255,0), ox+8+4+3+w, oy+8-4, 0, "%s, %s", al_keycode_to_name(value1), al_keycode_to_name(value2));
            al_draw_textf(fnt, al_map_rgb(255,0,0), ox+8+4+3+w, oy+8-4, 0, "%s", al_keycode_to_name(value1));
            if(is_disabled) al_draw_textf(fnt, al_map_rgba(0,0,0, 195), ox+8+4+3+w, oy+8-4, 0, "%s", al_keycode_to_name(value1));
        } else if(conflict2) {
            al_draw_textf(fnt, al_map_rgb(255,0,0), ox+8+4+3+w, oy+8-4, 0, "%s, %s", al_keycode_to_name(value1), al_keycode_to_name(value2));
            al_draw_textf(fnt, al_map_rgb(0,255,0), ox+8+4+3+w, oy+8-4, 0, "%s,", al_keycode_to_name(value1));
            if(is_disabled) al_draw_textf(fnt, al_map_rgba(0,0,0, 195), ox+8+4+3+w, oy+8-4, 0, "%s", al_keycode_to_name(value1));
        } else {
            al_draw_textf(fnt, al_map_rgb(0,255,0), ox+8+4+3+w, oy+8-4, 0, "%s, %s", al_keycode_to_name(value1), al_keycode_to_name(value2));
            if(is_disabled) al_draw_textf(fnt, al_map_rgba(0,0,0, 195), ox+8+4+3+w, oy+8-4, 0, "%s", al_keycode_to_name(value1));
        }
    }



	if(has_focus && partsys->topInteract==container) {
        //al_set_blender(ALLEGRO_DEST_MINUS_SRC, ALLEGRO_INVERSE_ALPHA, ALLEGRO_ONE);
	    int I = partsys->get_part_count();
		for( int i=0; i<I; i++ ) {
			double xx,yy;
			xx = partsys->get_part_x(i);
			yy = partsys->get_part_y(i);
			if(i%4==0) al_draw_filled_circle(ox-x+xx+lengthdir_x(10, i*1440/I),oy-y+yy+lengthdir_y(10, i*1440/I),10, al_map_rgba(0,127,0, 25));
			else if(i%2==0) al_draw_filled_circle(ox-x+xx+lengthdir_x(5, i*1440/I),oy-y+yy+lengthdir_y(5, i*1440/I),10, al_map_rgba(0,127,127, 25));
			else al_draw_filled_circle(ox-x+xx,oy-y+yy,10, al_map_rgba(0,255,0, 6));
		}
		//al_set_blender(ALLEGRO_ADD, ALLEGRO_ALPHA, ALLEGRO_INVERSE_ALPHA);
	}


}


// --~--


IGetJoy::IGetJoy(Interactive *con, IPartSystem *sys, int nx, int ny, int nw, int nh, int n, int a, bool p, bool s, int j, const char *nl, int index) :
    IButton(con, sys, nx, ny, nw, nh, nl, index) {

    isStick = s;
    num = n;
    axe = a;
    plus = p;
    jid = j;

    conflict = false;
    in_state = false;

    submit_id = 0;
    cancel_id = 0;

}

void IGetJoy::get_value(int& n, int& a, bool& p, bool& s, int& j) {
    n = num;
    a = axe;
    p = plus;
    s = isStick;
    j = jid;
}

void IGetJoy::set_values(int n, int a, bool p, bool s, int j) {
    isStick = s;
    num = n;
    axe = a;
    plus = p;
    jid = j;
}

void IGetJoy::set_submitCancel(int submit, int cancel) {
    submit_id = submit;
    cancel_id = cancel;
}

int IGetJoy::use(ActionManager& act, Keyboard& keys) {

    if(!has_focus) return 0;

    if(in_state) {

        bool up = keys.get_key1(KEY_TYPE_UP)->pressed();
        bool left = keys.get_key1(KEY_TYPE_LEFT)->pressed();
        bool right = keys.get_key1(KEY_TYPE_RIGHT)->pressed();
        bool down = keys.get_key1(KEY_TYPE_DOWN)->pressed();

        if(up || down || left || right
        || act.quit()) {

            in_state = false;
            return 0;

        }

        if(act.get_joy(jid)->get_any()->pressed()) {
            act.get_joy(jid)->get_anynum(num, axe, isStick, plus);
            in_state = false;
            return id;
        }


    } else {

        if(act.quit()) { return 0; }
        else if(act.UP()) container->set_used_but(adj_up);
        else if(act.DOWN()) container->set_used_but(adj_down);
        else if(act.LEFT()) container->set_used_but(adj_left);
        else if(act.RIGHT()) container->set_used_but(adj_right);
        else if(act.SELECT()) {
            return 0;
        }
        else if(act.ANY()) {

            in_state = true;
        }

    }

	return 0;


}

int IGetJoy::test(Joystick *joy) {

    conflict = false;
    int k = joy->conflict(isStick, num, axe, plus);
    if(k>=0) conflict = true;
    return k;
}

void IGetJoy::update() {

    if(!has_focus || partsys->topInteract!=container) return;

	// Particle control
	double t = partsys->get_part_time();
	int I = partsys->get_part_count();
    for( int i=0; i<I; i++ ) {

        double xx,yy;
        double j = ((double) i)/I;
        j+=t/1.0; // 1.0 rounds per second
        while(j>1.0) j-=1.0;
        while(j<0.0) j+=1.0;
        ol_to_coords(this, j, xx, yy);
        xx+=x;
        yy+=y;
        if(in_state) xx += w+3;
        partsys->set_part(i, xx,yy, 0,0);

    }
    // --

}

void IGetJoy::draw(int offx, int offy, ALLEGRO_FONT *fnt, int t) {

	int ox = offx+x;
	int oy = offy+y;

    al_draw_filled_rectangle(ox,oy,ox+w,oy+h,al_map_rgba(100,159,255,100));
    al_draw_filled_rectangle(ox+w+3,oy,ox+w+4+w,oy+h,al_map_rgba(0,0,0,100));

	al_draw_rectangle(ox,oy,ox+w,oy+h,al_map_rgba(155,155,55,100), 0.0);
	al_draw_rectangle(ox+w+3,oy,ox+w+4+w,oy+h,al_map_rgba(155,155,55,100), 0.0);


    ALLEGRO_COLOR txt_color, key_color;
    if(is_disabled) txt_color = al_map_rgb(0,0,0);
    else txt_color = al_map_rgb(0,0,0);

    if(conflict) key_color = al_map_rgb(255,0,0);
    else key_color = al_map_rgb(0,255,0);

    al_draw_text(fnt, txt_color, ox+8-4, oy+8-4, 0, label);
    if(in_state) {
        al_draw_text(fnt, key_color, ox+8+4+3+w, oy+8-4, 0, "<Press a button>");
    } else {
        const char *name = (isStick)? al_get_joystick_stick_name(al_get_joystick(jid), num) : al_get_joystick_button_name(al_get_joystick(jid), num) ;
        const char *stats;
        if(isStick) {
            if(axe==0 && plus) stats = "right" ;
            if(axe==0 && !plus) stats = "left" ;
            if(axe==1 && plus) stats = "down" ;
            if(axe==1 && !plus) stats = "up" ;
            if(axe==2 && plus) stats = "nearer" ;
            if(axe==2 && !plus) stats = "farther" ;
        } else stats="";

        if(conflict) al_draw_textf(fnt, al_map_rgb(255,0,0), ox+8+4+3+w, oy+8-4, 0, "%s %s", name, stats);
        else al_draw_textf(fnt, al_map_rgb(0,255,0), ox+8+4+3+w, oy+8-4, 0, "%s %s", name, stats);
        if(is_disabled) al_draw_textf(fnt, al_map_rgba(0,0,0, 195), ox+8+4+3+w, oy+8-4, 0, "%s %s", name, stats);
    }



	if(has_focus && partsys->topInteract==container) {
        //al_set_blender(ALLEGRO_DEST_MINUS_SRC, ALLEGRO_INVERSE_ALPHA, ALLEGRO_ONE);
	    int I = partsys->get_part_count();
		for( int i=0; i<I; i++ ) {
			double xx,yy;
			xx = partsys->get_part_x(i);
			yy = partsys->get_part_y(i);
			if(i%4==0) al_draw_filled_circle(ox-x+xx+lengthdir_x(10, i*1440/I),oy-y+yy+lengthdir_y(10, i*1440/I),10, al_map_rgba(0,127,0, 25));
			else if(i%2==0) al_draw_filled_circle(ox-x+xx+lengthdir_x(5, i*1440/I),oy-y+yy+lengthdir_y(5, i*1440/I),10, al_map_rgba(0,127,127, 25));
			else al_draw_filled_circle(ox-x+xx,oy-y+yy,10, al_map_rgba(0,255,0, 6));
		}
		//al_set_blender(ALLEGRO_ADD, ALLEGRO_ALPHA, ALLEGRO_INVERSE_ALPHA);
	}


}


// --~--


Interactive::Interactive(IPartSystem *sys, int nx, int ny, int nw, int nh, const char *nt) {

    partsys = sys;
    prevInteract = partsys->topInteract;
    partsys->topInteract = this;

	x = nx;
	y = ny;
	w = nw;
	h = nh;
	title = nt;

	button_in_use = 0;

    ResourceManager::resMan->add_resource("fnt/DAYROM__.ttf", FILE_TYPE_FNT, true, 14);
	fnt = ResourceManager::resMan->get_fnt("fnt/DAYROM__.ttf", 14);

	cycle_count = 0;
	cycle_delay = 20;

}

Interactive::~Interactive() {

    partsys->topInteract = prevInteract;


	// Iterate and delete
	for( int i=0; i<list_buttons.size(); i++ ) {
		delete list_buttons[i];
	}

}

int Interactive::get_x() {
    return x;
}
int Interactive::get_y() {
    return y;
}
int Interactive::get_w() {
    return w;
}
int Interactive::get_h() {
    return h;
}

void Interactive::set_x(int nx) {
    x = nx;
}
void Interactive::set_y(int ny) {
    y = ny;
}

void Interactive::set_w(int nw) {
    w = nw;
}
void Interactive::set_h(int nh) {
    h = nh;
}

const char * Interactive::get_title() {
    return title;
}

int Interactive::update(ActionManager& act, Keyboard& inp_keyboard) {

    // Particles
	button_in_use->update();
    // --

	if(act.released()) {
		cycle_count = cycle_delay+1;
	}

	if(cycle_count<=cycle_delay) {
		cycle_count++;
		return 0;
	}

	// Check for changes?
	// Update button with focus
	int used = 0;
	if(button_in_use!=0) {
		used = button_in_use->use(act, inp_keyboard);
	}

	if(act.held()) {
		cycle_count = 0;
	}

	return used;

}

IButton * Interactive::add_button(IPartSystem * sys, int nx, int ny, int nw, int nh, const char *nl, int index) {

	// Create a button
	IButton *but = new IButton(this, sys, nx,ny,nw,nh,nl, index);
	list_buttons.push_back(but);

	// Blur other
	set_used_but(but);

	// ...and return it
	return but;

}

IGetVal * Interactive::add_getval(IPartSystem * sys, int nx, int ny, int nw, int nh, int startv,int minv,int maxv, int index) {

	// Create a button
	IGetVal *but = new IGetVal(this, sys, nx,ny,nw,nh, startv,minv,maxv, index);
	list_buttons.push_back(but);

	// Blur other
	set_used_but(but);

	// ...and return it
	return but;

}

IGetKey * Interactive::add_getkey(IPartSystem * sys, int nx, int ny, int nw, int nh, int k1, int k2, const char *nl, int index) {

	// Create a button
	IGetKey *ik = new IGetKey(this, sys, nx,ny,nw,nh, k1, k2, nl, index);
	list_buttons.push_back(ik);

	// Blur other
	set_used_but(ik);

	// ...and return it
	return ik;

}

IGetJoy * Interactive::add_getjoy(IPartSystem * sys, int nx, int ny, int nw, int nh, int n, int a, bool p, bool s, int j, const char *nl, int index) {

	// Create a button
	IGetJoy *ij = new IGetJoy(this, sys, nx,ny,nw,nh, n, a, p, s, j, nl, index);
	list_buttons.push_back(ij);

	// Blur other
	set_used_but(ij);

	// ...and return it
	return ij;

}

void Interactive::set_used_but(IButton *but) {
	if(but!=0) {
		if(button_in_use!=0) button_in_use->pass_focus(but);
		button_in_use = but;
	}
}

IButton * Interactive::get_used_but() {
	return button_in_use;
}


void Interactive::set_initial_button(IButton *but) {

	// Blur last added button and focus this
	//list_buttons.back().pass_focus(but);
	//button_in_use = but;
	//set_used_but(but);

}

void Interactive::set_adjacent_buttons(IButton *button, IButton *up, IButton *left, IButton *right, IButton *down) {
	button -> setup_adj(up, left, right, down);
}

void Interactive::draw(int timing) {

	// Iterate and draw
	for( int i=0; i<list_buttons.size(); i++ ) {
		list_buttons[i]->draw(x,y, fnt, timing);
	}
	// Particles

}
