#include <iostream>
#include "joystick.h"

const char *Joystick::msg_text = "";
int Joystick::msg_count = 0;
int Joystick::msg_delay = 180;


Joystick::Joystick(ALLEGRO_JOYSTICK *identity) :
    any() {

    identifier = identity;
    inactive = !al_get_joystick_active(identity);

	int def[NUM_JOYBUTS];

	def[JOY_TYPE_ACT1] = 0;
	def[JOY_TYPE_ACT2] = 1;
	def[JOY_TYPE_ACT3] = 2;
	def[JOY_TYPE_ACT4] = 3;
	def[JOY_TYPE_LTRIGGER] = 4;
	def[JOY_TYPE_RTRIGGER] = 5;
	def[JOY_TYPE_START] = 9;
	def[JOY_TYPE_SELECT] = 8;
	def[JOY_TYPE_UP] = 0;
	def[JOY_TYPE_DOWN] = 0;
	def[JOY_TYPE_LEFT] = 1;
	def[JOY_TYPE_RIGHT] = 1;

	for( int i=0; i<NUM_JOYBUTS; i++ ) {
		btns[i].joyType = JOY_TYPE(i);
		btns[i].axisNum = 0;
		btns[i].ctrlNum = def[i];
		btns[i].isStick = false;
		btns[i].isPlus = false;
		btns[i].prev = 0.0f; // for debugging
	}

    btns[JOY_TYPE_UP].axisNum = 1;
    btns[JOY_TYPE_UP].ctrlNum = 0;
    btns[JOY_TYPE_UP].isStick = true;
    btns[JOY_TYPE_UP].isPlus = false;

    btns[JOY_TYPE_DOWN].axisNum = 1;
    btns[JOY_TYPE_DOWN].ctrlNum = 0;
    btns[JOY_TYPE_DOWN].isStick = true;
    btns[JOY_TYPE_DOWN].isPlus = true;

    btns[JOY_TYPE_LEFT].axisNum = 0;
    btns[JOY_TYPE_LEFT].ctrlNum = 0;
    btns[JOY_TYPE_LEFT].isStick = true;
    btns[JOY_TYPE_LEFT].isPlus = false;

    btns[JOY_TYPE_RIGHT].axisNum = 0;
    btns[JOY_TYPE_RIGHT].ctrlNum = 0;
    btns[JOY_TYPE_RIGHT].isStick = true;
    btns[JOY_TYPE_RIGHT].isPlus = true;

	anybtnctrl = 0;
	anybtnaxis = 0;
	anybtnstick = false;
	anybtnplus = false;

	// --~--

}

/*
bool operator == (Joystick param) {
    if(anybtn==param.anybtn) {
        std::cout << "comparing_joysticks (true)" << std::endl;
        Joystick::start_msg("Comparing joysticks (true). ");
        return true;
    } else {
        std::cout << "comparing_joysticks (false)" << std::endl;
        Joystick::start_msg("Comparing joysticks (false). ");
        return false;
    }

}
*/

ALLEGRO_JOYSTICK * Joystick::get_identity() {
    return identifier;
}

int Joystick::conflict(bool s, int n, int a, bool p) {

    for( int i=0; i<NUM_JOYBUTS; i++ ) {
        if(btns[i].isStick!=s || btns[i].ctrlNum!=n) continue;
        if(!s || (s && btns[i].axisNum==a && btns[i].isPlus==p)) {
            for( int j=0; j<NUM_JOYBUTS; j++ ) {
                if(i==j) continue;
                if(btns[j].isStick!=s || btns[j].ctrlNum!=n) continue;
                if(!s || (s && btns[j].axisNum==a && btns[j].isPlus==p)) {
                    return btns[j].joyType;
                }
            }
        }
    }

	return -1;
}

void Joystick::display_msg(ALLEGRO_FONT *fnt) {

    if(msg_count<=0) return;
    int numb = al_get_num_joysticks();
    al_draw_textf(fnt, al_map_rgb(0,0,0), 3,4,0, "%s Num joysticks: %d", msg_text, numb);
    al_draw_textf(fnt, al_map_rgb(0,0,0), 4,5,0, "%s Num joysticks: %d", msg_text, numb);
    al_draw_textf(fnt, al_map_rgb(0,0,0), 4,3,0, "%s Num joysticks: %d", msg_text, numb);
    al_draw_textf(fnt, al_map_rgb(0,0,0), 5,4,0, "%s Num joysticks: %d", msg_text, numb);
    al_draw_textf(fnt, al_map_rgb(0,255,0), 4,4,0, "%s Num joysticks: %d", msg_text, numb);
    msg_count--;

}

void Joystick::start_msg(const char *msg) {

    msg_text = msg;
    msg_count = msg_delay;

}

void Joystick::on_event(ALLEGRO_EVENT * ev) {

     if(ev->type == ALLEGRO_EVENT_JOYSTICK_AXIS) {
		if(ev->joystick.id==identifier) {
		    //std::cout << "AXIS: stick: " << ev->joystick.stick << " axis: " << ev->joystick.axis << " pos: " << ev->joystick.pos << std::endl;

            anybtnstick = true;
            if(ev->joystick.pos>=0.6f
            || ev->joystick.pos<=-0.6f) {
                anybtnctrl = ev->joystick.stick;
                anybtnaxis = ev->joystick.axis;
                any.update(true);
            }
            if(ev->joystick.pos<=0.4f
            || ev->joystick.pos>=-0.4f) {
                any.update(false);
            }
            if(ev->joystick.pos>=-0.4f) {
                anybtnplus = true;

            }
            if(ev->joystick.pos<=0.4f) {
                anybtnplus = false;
            }

            for( int i=0; i<NUM_JOYBUTS; i++ ) {
                if(!btns[i].isStick) continue;
                if(btns[i].ctrlNum==ev->joystick.stick
                && btns[i].axisNum==ev->joystick.axis) {

                    if(btns[i].isPlus && ev->joystick.pos>=-0.05) {
                        if(ev->joystick.pos>=0.6f) {
                            btns[i].state.update(true);
                            btns[i].prev = ev->joystick.pos;
                        }
                        if(ev->joystick.pos<=0.4f) {
                            btns[i].state.update(false);
                            btns[i].prev = ev->joystick.pos;
                        }
                    }
                    if(!btns[i].isPlus && ev->joystick.pos<=0.05) {
                        if(ev->joystick.pos<=-0.6f) {
                            btns[i].state.update(true);
                            btns[i].prev = ev->joystick.pos;
                        }
                        if(ev->joystick.pos>=-0.4f) {
                            btns[i].state.update(false);
                            btns[i].prev = ev->joystick.pos;
                        }
                    }

                }
            }
		}
	} else if(ev->type == ALLEGRO_EVENT_JOYSTICK_BUTTON_DOWN) {
	    //std::cout << "PRESS: stick: " << ev->joystick.stick << " button: " << ev->joystick.button << std::endl;
        if(ev->joystick.id==identifier) {
            any.update(true);
            anybtnctrl = ev->joystick.button;
            anybtnstick = false;
            for( int i=0; i<NUM_JOYBUTS; i++ ) {
                if(btns[i].isStick) continue;
                if(btns[i].ctrlNum==ev->joystick.button) btns[i].state.update(true);
            }
        }
	} else if(ev->type == ALLEGRO_EVENT_JOYSTICK_BUTTON_UP) {
		if(ev->joystick.id==identifier) {
            any.update(false);
            anybtnstick = false;
            for( int i=0; i<NUM_JOYBUTS; i++ ) {
                if(btns[i].isStick) continue;
                if(btns[i].ctrlNum==ev->joystick.button) btns[i].state.update(false);
            }
		}
	}
}

void Joystick::on_timer() {
	any.sync();
	for( int i=0; i<NUM_JOYBUTS; i++ ) {
		btns[i].state.sync();
	}
}

void Joystick::on_blur() {
	any.update(false);
	for( int i=0; i<NUM_JOYBUTS; i++ ) {
		btns[i].state.update(false);
	}
}

void Joystick::set_joy(JOY_TYPE typ, int bn) {
	btns[typ].ctrlNum = bn;
	btns[typ].axisNum = 0;
	btns[typ].isPlus = false;
	btns[typ].isStick = false;
	//std::cout << "RECONFIGURATION: stick: " << btns[typ].ctrlNum << " axis: " << btns[typ].axisNum << " is plus: " << btns[typ].isPlus << " is stick: " << btns[typ].isStick << std::endl;
}

void Joystick::set_axis(JOY_TYPE typ, int an, int axis, bool plus) {
    btns[typ].ctrlNum = an;
    btns[typ].axisNum = axis;
    btns[typ].isPlus = plus;
    btns[typ].isStick = true;
    //std::cout << "RECONFIGURATION: stick: " << btns[typ].ctrlNum << " axis: " << btns[typ].axisNum << " is plus: " << btns[typ].isPlus << " is stick: " << btns[typ].isStick << std::endl;
}

Control * Joystick::get_btn(JOY_TYPE typ) {
	return &btns[typ].state;
}

const char * Joystick::get_btnname(JOY_TYPE typ) {
	return al_get_joystick_button_name(identifier, btns[typ].ctrlNum);
}

Control * Joystick::get_any() {
	return &any;
}

void Joystick::get_anynum(int& nctrl, int& naxis, bool& stick, bool& plus) {
	nctrl = anybtnctrl;
	naxis = anybtnaxis;
	stick = anybtnstick;
	plus = anybtnplus;
}

int Joystick::get_bntnum(JOY_TYPE typ, int& nctrl, int& naxis, bool& stick, bool& plus) {

    nctrl = btns[typ].ctrlNum;
	naxis = btns[typ].axisNum;
	stick = btns[typ].isStick;
	plus = btns[typ].isPlus;

	return btns[typ].ctrlNum;
}
