#ifndef JOYSTICK_H_INCLUDED
#define JOYSTICK_H_INCLUDED

#include <allegro5/allegro.h>
#include <allegro5/allegro_image.h>
#include <allegro5/allegro_color.h>
#include <allegro5/allegro_font.h>
#include <allegro5/allegro_ttf.h>
#include <allegro5/allegro_primitives.h>
#include "control.h"

const int NUM_JOYBUTS = 12;

enum JOY_TYPE {
	JOY_TYPE_START,
	JOY_TYPE_SELECT,
	JOY_TYPE_ACT1,
	JOY_TYPE_ACT2,
	JOY_TYPE_ACT3,
	JOY_TYPE_ACT4,
	JOY_TYPE_LTRIGGER,
	JOY_TYPE_RTRIGGER,
	JOY_TYPE_UP,
	JOY_TYPE_DOWN,
    JOY_TYPE_LEFT,
	JOY_TYPE_RIGHT
};

struct Joy {
	JOY_TYPE joyType;
	int axisNum;
	int ctrlNum;
	bool isStick;
	bool isPlus;
	float prev;
	Control state;
};

class Joystick {

	public:

	// Fill in keys already known
	Joystick(ALLEGRO_JOYSTICK *identity);

	// Keep things updated
	void on_event(ALLEGRO_EVENT * ev);
	void on_timer();
	void on_blur();

	// Change the actual joystick button for this type
	void set_joy(JOY_TYPE typ, int bn);
	void set_axis(JOY_TYPE typ, int an, int axis, bool plus);
	int get_bntnum(JOY_TYPE typ, int& nctrl, int& naxis, bool& stick, bool& plus);
	// Return a pointer to a control with the active state of the button
	Control * get_btn(JOY_TYPE typ);
	const char * get_btnname(JOY_TYPE typ);

    int conflict(bool s, int n, int a, bool p);
	Control * get_any();
	void get_anynum(int& nctrl, int& naxis, bool& stick, bool& plus);


    static void display_msg(ALLEGRO_FONT *fnt);
    static void start_msg(const char *msg);

    //bool operator == (Joystick);
    bool inactive;
    ALLEGRO_JOYSTICK * get_identity();


	private:

    static const char *msg_text;
    static int msg_count;
    static int msg_delay;

    // Handle
    ALLEGRO_JOYSTICK *identifier;
	// List of valid keys to use in a certain situation
	Joy btns[NUM_JOYBUTS];
	Control any;
	int anybtnctrl;
	int anybtnaxis;
	bool anybtnstick;
	bool anybtnplus;

};

#endif // JOYSTICK_H_INCLUDED
