#include "key.h"

Keyboard::Keyboard() : any() {

	int def[NUM_KEYS];

	def[KEY_TYPE_QUIT] = ALLEGRO_KEY_ESCAPE; // or ALLEGRO_KEY_Q;
	def[KEY_TYPE_START] = ALLEGRO_KEY_PAD_5; // or ALLEGRO_KEY_PAD_ENTER
	def[KEY_TYPE_SELECT] = ALLEGRO_KEY_BACKSPACE;
	def[KEY_TYPE_ACT1] = ALLEGRO_KEY_I;
	def[KEY_TYPE_ACT2] = ALLEGRO_KEY_J;
	def[KEY_TYPE_ACT3] = ALLEGRO_KEY_K;
	def[KEY_TYPE_ACT4] = ALLEGRO_KEY_L;
	def[KEY_TYPE_LTRIGGER] = ALLEGRO_KEY_U;
	def[KEY_TYPE_RTRIGGER] = ALLEGRO_KEY_O;
	def[KEY_TYPE_UP] = ALLEGRO_KEY_PAD_8;
	def[KEY_TYPE_DOWN] = ALLEGRO_KEY_PAD_2;
	def[KEY_TYPE_LEFT] = ALLEGRO_KEY_PAD_4;
	def[KEY_TYPE_RIGHT] = ALLEGRO_KEY_PAD_6;

	for( int i=0; i<NUM_KEYS; i++ ) {
		keys2[i].keyType = KEY_TYPE(i);
		keys2[i].keyCode = def[i];
	}

	def[KEY_TYPE_QUIT] = ALLEGRO_KEY_ESCAPE;
	def[KEY_TYPE_START] = ALLEGRO_KEY_ENTER;
	def[KEY_TYPE_SELECT] = ALLEGRO_KEY_TAB;
	def[KEY_TYPE_ACT1] = ALLEGRO_KEY_S;
	def[KEY_TYPE_ACT2] = ALLEGRO_KEY_Z;
	def[KEY_TYPE_ACT3] = ALLEGRO_KEY_X;
	def[KEY_TYPE_ACT4] = ALLEGRO_KEY_C;
	def[KEY_TYPE_LTRIGGER] = ALLEGRO_KEY_A;
	def[KEY_TYPE_RTRIGGER] = ALLEGRO_KEY_D;
	def[KEY_TYPE_UP] = ALLEGRO_KEY_UP;
	def[KEY_TYPE_DOWN] = ALLEGRO_KEY_DOWN;
	def[KEY_TYPE_LEFT] = ALLEGRO_KEY_LEFT;
	def[KEY_TYPE_RIGHT] = ALLEGRO_KEY_RIGHT;

	for( int i=0; i<NUM_KEYS; i++ ) {
		keys1[i].keyType = KEY_TYPE(i);
		keys1[i].keyCode = def[i];
	}

	anycode = ALLEGRO_KEY_SPACE;

}


KEY_TYPE Keyboard::conflict(int code) {

    KEY_TYPE same = KEY_TYPE_QUIT;

    for( int i=0; i<NUM_KEYS; i++ ) {
        if(keys1[i].keyCode != code
        && keys2[i].keyCode != code) {
            continue;
        }
        for( int j=0; j<NUM_KEYS; j++ ) {
            if(i==j) continue;
            if(keys1[j].keyCode == code
            || keys2[j].keyCode == code) {
                same=keys2[j].keyType;
            }
        }
	}

	return same;
}


void Keyboard::on_event(ALLEGRO_EVENT * ev) {
	if(ev->type == ALLEGRO_EVENT_KEY_DOWN) {
		any.update(true);
		anycode = ev->keyboard.keycode;
		for( int i=0; i<NUM_KEYS; i++ ) {
			if(keys1[i].keyCode==ev->keyboard.keycode) keys1[i].state.update(true);
			if(keys2[i].keyCode==ev->keyboard.keycode) keys2[i].state.update(true);
		}
	} else if(ev->type == ALLEGRO_EVENT_KEY_UP) {
		any.update(false);
		for( int i=0; i<NUM_KEYS; i++ ) {
			if(keys1[i].keyCode==ev->keyboard.keycode) keys1[i].state.update(false);
			if(keys2[i].keyCode==ev->keyboard.keycode) keys2[i].state.update(false);
		}
	}
}

void Keyboard::on_timer() {
	any.sync();
	for( int i=0; i<NUM_KEYS; i++ ) {
		keys1[i].state.sync();
		keys2[i].state.sync();
	}
}

void Keyboard::on_blur() {
	any.update(false);
	for( int i=0; i<NUM_KEYS; i++ ) {
		keys1[i].state.update(false);
		keys2[i].state.update(false);
	}
}

void Keyboard::set_key(KEY_TYPE key, int code1, int code2) {
	keys1[key].keyCode = code1;
	keys2[key].keyCode = code2;
}

Control * Keyboard::get_key1(KEY_TYPE key) {
	return &keys1[key].state;
}

Control * Keyboard::get_key2(KEY_TYPE key) {
	return &keys2[key].state;
}

const char * Keyboard::get_keyname1(KEY_TYPE key) {
	return al_keycode_to_name(keys1[key].keyCode);
}

const char * Keyboard::get_keyname2(KEY_TYPE key) {
	return al_keycode_to_name(keys2[key].keyCode);
}

Control * Keyboard::get_any() {
	return &any;
}

int Keyboard::get_anycode() {
	return anycode;
}

int Keyboard::get_keycode1(KEY_TYPE key) {
	return keys1[key].keyCode;
}

int Keyboard::get_keycode2(KEY_TYPE key) {
	return keys2[key].keyCode;
}
