#include "level.h"

Level::Level(int num_cols, int num_rows) {

    cols = num_cols;
    rows = num_rows;

    cellw = 32.0;
    cellh = 32.0;

    // Init grid
    grid = new Obstacle**[rows];
    for( int r=0; r<rows; r++ ) grid[r] = new Obstacle*[cols];

	// Fill grid with null pointers
	for( int r=0; r<rows; r++ ) {
        for( int c=0; c<cols; c++ ) {
            grid[r][c] = 0;
        }
    }

}

Level::~Level() {

    //Delete all obstacles
    for( int r=0; r<rows; r++ ) {
        for( int c=0; c<cols; c++ ) {
            if(grid[r][c]==0) continue;
            delete grid[r][c];
            grid[r][c] = 0;
        }
    }


	//Deallocate grid
	for( int r=0; r<rows; r++ ) delete [] grid[r];
	delete [] grid;
}

int Level::get_cols() {

    return cols;

}

int Level::get_rows() {

    return rows;

}

double Level::get_width() {

    return cellw;

}

double Level::get_height() {

    return cellh;

}

double Level::get_total_width() {

    return cols*cellw;

}

double Level::get_total_height() {

    return rows*cellh;

}


Obstacle * Level::add_obst(double xpos, double ypos) {

    int col, row;
        col = xpos / cellw;
        row = ypos / cellh;

    if(col<0 || row<0 || col>=cols || row>=rows) return 0;
    if(grid[row][col]==0) {
        Obstacle *it = new Obstacle(col, row);
        grid[row][col] = it;
        return it;
    }

    return 0;
}

void Level::rem_obst(Obstacle *obst) {

    if(obst==0) return;

    grid[obst->get_r()][obst->get_c()] = 0;
    delete obst;

}

Obstacle * Level::get_obst(int c, int r) {
    if(c<0 || r<0 || c>=cols || r>=rows) return 0;
    return grid[r][c];
}
