#include <stdio.h>
#include <iostream>
#include <allegro5/allegro.h>
#include <allegro5/allegro_image.h>
//#include <allegro5/allegro_physfs.h>
#include <allegro5/allegro_primitives.h>
#include <allegro5/allegro_color.h>
#include <allegro5/allegro_font.h>
#include <allegro5/allegro_ttf.h>
#include <allegro5/allegro_physfs.h>
#include <allegro5/allegro_audio.h>
#include <allegro5/allegro_acodec.h>
#include <physfs.h>
#include "configure.h"
#include "application.h"

struct Assets {
	ALLEGRO_DISPLAY *display;
	ALLEGRO_EVENT_QUEUE *event_queue;
	ALLEGRO_TIMER *timer;
	Settings *config;
};

bool init( Assets& v );
bool setup( Assets& v );
bool run( Assets& v );
bool deinit( Assets& v );

int main(int argc, char **argv) {

	Assets vars;
		vars.display = 0;
		vars.event_queue = 0;
		vars.timer = 0;
		vars.config = 0;

	if(init(vars)) {

		setup(vars);
		run(vars);
		deinit(vars);

		return 0;
	}

	return 1;

}


bool init( Assets& v ) {

	//ESSENTIALS
	if(!al_init()) {
		fprintf(stderr, "failed to initialize allegro!\n");
	} else if(!al_install_keyboard()) {
		fprintf(stderr, "failed to initialize the keyboard!\n");
	} else if(!al_install_joystick()) {
		fprintf(stderr, "failed to initialize the joystick!\n");
	} else if(!al_install_mouse()) {
		fprintf(stderr, "failed to initialize the mouse!\n");
	} else if(!al_install_audio()) {
		fprintf(stderr, "failed to initialize audio!\n");
	} else {

		//ASSETS
		//al_set_new_display_flags(ALLEGRO_FULLSCREEN); //tryme
		//al_set_new_display_flags(ALLEGRO_RESIZABLE); //tryme
		al_set_new_display_option(ALLEGRO_SAMPLE_BUFFERS, 1, ALLEGRO_SUGGEST);
		al_set_new_display_option(ALLEGRO_SAMPLES, 4, ALLEGRO_SUGGEST);

		v.config = new Settings();
		if(v.config->get_fullscreen()) al_set_new_display_flags(ALLEGRO_FULLSCREEN_WINDOW);

		v.display = al_create_display(v.config -> get_screen_w(), v.config -> get_screen_h());
		v.timer = al_create_timer(1.0 / v.config -> FPS);
		v.event_queue = al_create_event_queue();
		if(!v.display) {
			fprintf(stderr, "failed to create display!\n");
		} else if(!v.timer) {
			fprintf(stderr, "failed to create timer!\n");
		} else if(!v.event_queue) {
			fprintf(stderr, "failed to create event_queue!\n");
		} else {

			al_init_font_addon();

			//ADDONS
			if(!al_init_image_addon()) {
				fprintf(stderr, "failed to initialize image addon!\n");
			} else if(!al_init_primitives_addon()) {
				fprintf(stderr, "failed to initialize primitive addon!\n");
			} else if(!al_init_ttf_addon()) {
				fprintf(stderr, "failed to initialize ttf font addon!\n");
			} else {

				//SUCCESS
				al_set_window_title(v.display, "SpectreNectar's WIP Game Title");
				al_set_target_bitmap(al_get_backbuffer(v.display));

				return true;

			}
		}

		deinit(v);

	}

	return false;

}

bool setup( Assets& v ) {

    /* Set up PhysicsFS. */
    if(!PHYSFS_init(0)) std::cout << "failed initializing" << std::endl;
    // This creates a ~/.allegro directory, which is very annoying to say the
    // least - and no need for it in this example.
    //  if (!PHYSFS_setSaneConfig("allegro", "ex_physfs", NULL, 0, 0))
    //     return 1;
    else {

        al_set_standard_file_interface();
        al_set_physfs_file_interface();

        if(!PHYSFS_addToSearchPath("data", 1)) std::cout << "failed adding search path" << std::endl;
        if(!PHYSFS_addToSearchPath("lvl", 1)) std::cout << "failed adding search path2" << std::endl;
    }

	al_register_event_source(v.event_queue, al_get_display_event_source(v.display));
	al_register_event_source(v.event_queue, al_get_timer_event_source(v.timer));
	al_register_event_source(v.event_queue, al_get_mouse_event_source());
	al_register_event_source(v.event_queue, al_get_keyboard_event_source());
	al_register_event_source(v.event_queue, al_get_joystick_event_source());

    // Audio
    al_reserve_samples(16);
    al_init_acodec_addon();

    //al_set_new_bitmap_flags(ALLEGRO_MIN_LINEAR | ALLEGRO_MAG_LINEAR); // Make full screen blurry

	al_set_blender(ALLEGRO_ADD, ALLEGRO_ALPHA, ALLEGRO_INVERSE_ALPHA); // Enable alpha for lines
	al_clear_to_color(al_map_rgb(0,0,0));
	al_flip_display();
	al_start_timer(v.timer);

    // Disable screensaver
    //al_inhibit_screensaver(true); //Full screen only
    //al_set_display_icon(v.display, bmp);
    //al_toggle_display_flag(v.display, ALLEGRO_FULLSCREEN_WINDOW, true);

	return true;
}

bool run( Assets& v ) {

	Application app(v.display, v.config);
	while(!app.doexit) {
		ALLEGRO_EVENT ev;
		al_wait_for_event(v.event_queue, &ev);

		app.update(&ev);

		if(app.redraw && al_is_event_queue_empty(v.event_queue)) {

			app.draw();
			al_flip_display();
		}
	}

	return true;
}

bool deinit( Assets& v ) {

    PHYSFS_deinit();

    delete v.config;
	al_uninstall_audio();
	al_destroy_timer(v.timer);
	al_destroy_event_queue(v.event_queue);
	al_destroy_display(v.display);

	return true;

}



