#include "mainMenu.h"


//Application *MainMenu::app = 0;

MainMenu::MainMenu(IPartSystem *sys, ActionManager& act) :
    RootMenu(sys, 130,40,320,180, "Main Menu") {

    // ...
    fullscreen = app -> config -> get_fullscreen();

	// Standard dialog
    in_dialog = false;
    dialog_id = 0;
    dialog = 0;
    // --

	but_startgame = interact.add_button(sys, 0,50,320,35, "Continue Game", MAINMENU_BUTTON_ID_STARTGAME);
	but_keyboard = interact.add_button(sys, 0,100,150,35, "Keyboard", MAINMENU_BUTTON_ID_KEYBOARD);
	but_joystick = interact.add_button(sys, 170,100,150,35, "Joystick", MAINMENU_BUTTON_ID_JOYSTICK);
	but_help = interact.add_button(sys, 0,150,320,35, "Help (F1)", MAINMENU_BUTTON_ID_HELP);
	but_quit = interact.add_button(sys, 0,200,320,35, "Quit (ESC)", MAINMENU_BUTTON_ID_QUIT);
	but_fullscreen = interact.add_button(sys, 340,50,150,35, "Fullscreen (F4)", MAINMENU_BUTTON_ID_FULLSCREEN);

    act.manage_joysticks();
    if(act.get_num_joys()==0) but_joystick->disable();

	interact.set_adjacent_buttons(but_startgame, 0, 0, but_fullscreen, but_keyboard);
	interact.set_adjacent_buttons(but_keyboard, but_startgame, 0, but_joystick, but_help);
	interact.set_adjacent_buttons(but_joystick, but_startgame, but_keyboard, 0, but_help);
	interact.set_adjacent_buttons(but_help, but_keyboard, 0, 0, but_quit);
	interact.set_adjacent_buttons(but_quit, but_help, 0, 0, 0);
	interact.set_adjacent_buttons(but_fullscreen, 0, but_startgame, 0, 0);

	interact.set_used_but(but_startgame);

	if(!fullscreen) {
        but_fullscreen->set_label("Fullscreen (F4)");
    } else {
        but_fullscreen->set_label("Windowed (F3)");
    }

}

MainMenu::~MainMenu() {
    if(dialog!=0) delete dialog;
}

void MainMenu::fun_startgame() {

	//app -> set_menu(APP_MENU_ID_NEWGAME);
    app -> set_app_state(APP_STATE_GAME_INIT, true);
}

void MainMenu::fun_keyboard() {

    app -> set_menu(APP_MENU_ID_OPTKEYS);

}

void MainMenu::fun_joystick(ActionManager& act) {

    act.manage_joysticks();
    //Joystick::start_msg("Try it - ");

    //Test if joystick is plugged in
    if(al_get_num_joysticks()>0) {

        app -> set_menu(APP_MENU_ID_OPTJOYS);

    } else {
        dialog_id = MAINMENU_DIALOG_ID_NOJOYSTICK;
        in_dialog = true;
        dialog = new DialogMenu(partsys, "No joystick plugged in.", false);
    }

}

void MainMenu::fun_help() {

	app -> set_app_state(APP_STATE_QUIT, false);

}

void MainMenu::fun_quit() {

	app -> set_app_state(APP_STATE_QUIT, false);

}

void MainMenu::fun_fullscreen() {

    if(app -> config -> get_fullscreen()) {
        but_fullscreen->set_label("Fullscreen (F4)");
        app -> config -> set_fullscreen(false);
        app -> fullscreen(false);
    } else {
        but_fullscreen->set_label("Windowed (F3)");
        app -> config -> set_fullscreen(true);
        app -> fullscreen(true);
    }

}


void MainMenu::update(ActionManager& act, Keyboard& inp_keyboard) {

    if(in_dialog) {

        // Prompt action
        switch(dialog_id) {
            case MAINMENU_DIALOG_ID_NOJOYSTICK:
            default:
                dialog->update(act, inp_keyboard);
            break;
        }

        if(dialog->done()) {
            delete dialog;
            dialog = 0;
            dialog_id = 0;
            in_dialog = false;
        }

    } else {

        // Spin
        int index = interact.update(act, inp_keyboard);

        switch(index) {
            case MAINMENU_BUTTON_ID_STARTGAME:
                fun_startgame();
            break;
            case MAINMENU_BUTTON_ID_KEYBOARD:
                fun_keyboard();
            break;
            case MAINMENU_BUTTON_ID_JOYSTICK:
                fun_joystick(act);
            break;
            case MAINMENU_BUTTON_ID_HELP:
                fun_help();
            break;
            case MAINMENU_BUTTON_ID_QUIT:
                fun_quit();
            break;
            case MAINMENU_BUTTON_ID_FULLSCREEN:
                fun_fullscreen();
            break;
        }

        if(act.quit()) {

            app -> set_app_state(APP_STATE_QUIT, false);
        }

    }

}

void MainMenu::draw(long anim_time) {

    for(int y=0; y<app->config->get_screen_h(); y+=al_get_bitmap_height(menuBG)*2) {
        for(int x=0; x<app->config->get_screen_w(); x+=al_get_bitmap_width(menuBG)*2) {
            al_draw_bitmap(menuBG, x, y, 0);
            al_draw_bitmap(menuBG, x+al_get_bitmap_width(menuBG), y, ALLEGRO_FLIP_HORIZONTAL);
            al_draw_bitmap(menuBG, x, y+al_get_bitmap_height(menuBG), ALLEGRO_FLIP_VERTICAL);
            al_draw_bitmap(menuBG, x+al_get_bitmap_width(menuBG), y+al_get_bitmap_height(menuBG), ALLEGRO_FLIP_HORIZONTAL | ALLEGRO_FLIP_VERTICAL);
        }
    }

	interact.draw(anim_time);

	if(in_dialog) {
        dialog->draw(anim_time);
    }

}
