#include "overlap.h"

// ~ Determine if two bitmaps collide

bool overlaps(ALLEGRO_BITMAP *bmp1, int b1_x1, int b1_y1, int b1_x2, int b1_y2, int b1_xoff, int b1_yoff,
              ALLEGRO_BITMAP *bmp2, int b2_x1, int b2_y1, int b2_x2, int b2_y2, int b2_xoff, int b2_yoff) {

    // First rule out cases where bounding boxes separate each other
    if(b1_x2 > b2_x1
    && b1_y2 > b2_y1
    && b2_x2 > b1_x1
    && b2_y2 > b1_y1) {

        // Region of overlap
        int ww = minimum(b1_x2, b2_x2)-maximum(b1_x1, b2_x1);
        int hh = minimum(b1_y2, b2_y2)-maximum(b1_y1, b2_y1);

        // Overlap offset in first bitmap
        int x1 = b1_xoff+maximum(b1_x1,b2_x1)-(b1_x1);
        int y1 = b1_yoff+maximum(b1_y1,b2_y1)-(b1_y1);
        // Overlap offset in second bitmap
        int x2 = b2_xoff+maximum(b1_x1,b2_x1)-(b2_x1);
        int y2 = b2_yoff+maximum(b1_y1,b2_y1)-(b2_y1);

        // Speed up reading from the bitmaps (further down)
        al_lock_bitmap_region(bmp1, x1, y1, ww, hh, al_get_bitmap_format(bmp1), ALLEGRO_LOCK_READONLY);
        al_lock_bitmap_region(bmp2, x2, y2, ww, hh, al_get_bitmap_format(bmp2), ALLEGRO_LOCK_READONLY);

        // Vars for values to read
        ALLEGRO_COLOR cc;
        unsigned char rr,gg,bb,aa;

        // X/Y to stop reading at
        int xx, yy;
            xx = x1+ww;
            yy = y1+hh;

        // Begin
        for( ; y1<=yy; y1++, y2++ ) {

            for( ; x1<=xx; x1++, x2++ ) {

                // Read from one bitmap
                cc = al_get_pixel(bmp2, x2, y2);
                al_unmap_rgba(cc, &rr,&gg,&bb,&aa);

                // If transparent then don't bother doing more
                if(aa==0) continue;

                // Read from other bitmap
                cc = al_get_pixel(bmp1, x1, y1);
                al_unmap_rgba(cc, &rr,&gg,&bb,&aa);

                if(aa!=0) {

                    // Always remember to unlock
                    al_unlock_bitmap(bmp1);
                    al_unlock_bitmap(bmp2);

                    //Collision!
                    return true;
                }


            }
            x1-=ww; //Reset
            x2-=ww; //Reset
        }
        // End

        // Always remember to unlock ;-)
        al_unlock_bitmap(bmp1);
        al_unlock_bitmap(bmp2);
    }

    return false;
}
