#include "resources.h"

ResourceManager *ResourceManager::resMan = 0;

ResourceManager::ResourceManager() {}

ResourceManager::~ResourceManager() {

    std::map<std::string , BMP_resource>::iterator bit;
    for( bit=map_bitmaps.begin(); bit!=map_bitmaps.end(); bit++ ) {
        al_destroy_bitmap((*bit).second.gfx);
    }

    std::map<std::string , SND_resource>::iterator sit;
    for( sit=map_sounds.begin(); sit!=map_sounds.end(); sit++ ) {
        al_destroy_sample((*sit).second.snd);
    }

    std::map<std::string , FNT_resource>::iterator fit;
    for( fit=map_fonts.begin(); fit!=map_fonts.end(); fit++ ) {
        al_destroy_font((*fit).second.fnt);
    }

    map_bitmaps.clear();
    map_sounds.clear();
    map_fonts.clear();

}


ALLEGRO_BITMAP * ResourceManager::get_bmp(const std::string& path) {
    std::map<std::string , BMP_resource>::iterator it = map_bitmaps.find(path);
    if(it!=map_bitmaps.end()) {
        aquire(path, FILE_TYPE_BMP);
        return map_bitmaps[path].gfx;
    }
    else return 0;
}

ALLEGRO_SAMPLE * ResourceManager::get_snd(const std::string& path) {
    std::map<std::string , SND_resource>::iterator it = map_sounds.find(path);
    if(it!=map_sounds.end()) {
        aquire(path, FILE_TYPE_SND);
        return map_sounds[path].snd;
    }
    else return 0;
}

ALLEGRO_FONT * ResourceManager::get_fnt(const std::string& path, int fnt_size) {
    std::map<std::string , FNT_resource>::iterator it = map_fonts.find(path);
    if(it!=map_fonts.end()) {
        aquire(path, FILE_TYPE_FNT, fnt_size);
        return map_fonts[path].fnt;
    }
    else return 0;
}


bool ResourceManager::add_resource(const std::string& path, FILE_TYPE typ, bool do_aquire, int fnt_size) {

    bool success = false;

    switch(typ) {
        case FILE_TYPE_BMP:
            {

                // One name for each
                std::map<std::string , BMP_resource>::iterator it = map_bitmaps.find(path);
                if(it!=map_bitmaps.end()) return true;

                map_bitmaps[path].isInitialized = false;
                map_bitmaps[path].isLoaded = false;
                map_bitmaps[path].gfx = 0;
                map_bitmaps[path].w = 0;
                map_bitmaps[path].h = 0;

                if(do_aquire) {
                    aquire(path, typ);
                }

                //map_bitmaps[path] = bmp;
                success = map_bitmaps[path].isLoaded;

            }
        break;
        case FILE_TYPE_SND:
            {

                // One name for each
                std::map<std::string , SND_resource>::iterator it = map_sounds.find(path);
                if(it!=map_sounds.end()) return true;

                map_sounds[path].is_aquired = false;
                map_sounds[path].snd = 0;

                if(do_aquire) {
                    aquire(path, typ);
                }

                //map_sounds[path] = snd;
                success = map_sounds[path].is_aquired;

            }
        break;
        case FILE_TYPE_FNT:
            {

                // One name for each
                std::map<std::string , FNT_resource>::iterator it = map_fonts.find(path);
                if(it!=map_fonts.end()) return true;

                map_fonts[path].is_aquired = false;
                map_fonts[path].fnt = 0;

                if(do_aquire) {
                    aquire(path, typ, fnt_size);
                }

                //map_fonts[path] = fnt;
                success = map_fonts[path].is_aquired;

            }
        break;
    }

    return success;

}

void ResourceManager::remove_resource(const std::string& path, FILE_TYPE typ) {

    unaquire(path, typ);

    switch(typ) {
        case FILE_TYPE_BMP:
            {
                std::map<std::string , BMP_resource>::iterator it = map_bitmaps.find(path);
                if(it!=map_bitmaps.end()) map_bitmaps.erase(it);
            }
        break;
        case FILE_TYPE_SND:
            {
                std::map<std::string , SND_resource>::iterator it = map_sounds.find(path);
                if(it!=map_sounds.end()) map_sounds.erase(it);
            }
        break;
        case FILE_TYPE_FNT:
            {
                std::map<std::string , FNT_resource>::iterator it = map_fonts.find(path);
                if(it!=map_fonts.end()) map_fonts.erase(it);
            }
        break;
    }

}

bool ResourceManager::aquire(const std::string& path, FILE_TYPE typ, int fnt_size) {

    switch(typ) {
        case FILE_TYPE_BMP:
            {

                std::map<std::string , BMP_resource>::iterator it = map_bitmaps.find(path);
                if(it==map_bitmaps.end()) return false;

                ALLEGRO_BITMAP *bmp = al_load_bitmap(path.c_str());

                if(!bmp) {
                    bmp = al_create_bitmap(32,32);
                    if(bmp) {

                        ALLEGRO_BITMAP * temp = al_get_target_bitmap();
                        al_set_target_bitmap(bmp);

                        al_clear_to_color(al_map_rgb(255, 255, 0));

                        al_set_target_bitmap(temp);

                        map_bitmaps[path].gfx = bmp;
                        map_bitmaps[path].isInitialized = true;
                    }
                    std::cout << "failed loading bitmap" << std::endl;
                    map_bitmaps[path].isLoaded = false;
                } else {
                    map_bitmaps[path].gfx = bmp;
                    map_bitmaps[path].isLoaded = true;
                    map_bitmaps[path].isInitialized = true;
                    al_convert_mask_to_alpha(map_bitmaps[path].gfx, al_map_rgb(255,0,255));
                }

                if(map_bitmaps[path].isInitialized) {
                    map_bitmaps[path].w = al_get_bitmap_width(map_bitmaps[path].gfx);
                    map_bitmaps[path].h = al_get_bitmap_height(map_bitmaps[path].gfx);
                }

                return map_bitmaps[path].isLoaded;

            }
        break;
        case FILE_TYPE_SND:
            {

                std::map<std::string , SND_resource>::iterator it = map_sounds.find(path);
                if(it==map_sounds.end()) return false;

                ALLEGRO_SAMPLE *sound = al_load_sample(path.c_str());

                if(!sound) {
                    ///TODO: Possibly attempt to load another sound
                    // For now just set to false
                    std::cout << "failed loading sound" << std::endl;
                    map_sounds[path].is_aquired = false;
                } else {
                    map_sounds[path].snd = sound;
                    map_sounds[path].is_aquired = true;
                }

                return map_sounds[path].is_aquired;

            }
        break;
        case FILE_TYPE_FNT:
            {

                std::map<std::string , FNT_resource>::iterator it = map_fonts.find(path);
                if(it==map_fonts.end()) return false;

                ALLEGRO_FONT *font = al_load_ttf_font(path.c_str(), fnt_size, 0);

                if(!font) {
                    ///TODO: Possibly attempt to load another font
                    // For now just set to false
                    std::cout << "failed loading font" << std::endl;
                    map_fonts[path].is_aquired = false;
                } else {
                    map_fonts[path].fnt = font;
                    map_fonts[path].is_aquired = true;
                }

                return map_fonts[path].is_aquired;

            }
        break;
    }

    return false;
}

void ResourceManager::unaquire(const std::string& path, FILE_TYPE typ) {


    switch(typ) {
        case FILE_TYPE_BMP:
            {

                std::map<std::string , BMP_resource>::iterator it = map_bitmaps.find(path);
                if(it==map_bitmaps.end()) return;

                if(map_bitmaps[path].isLoaded) al_destroy_bitmap(map_bitmaps[path].gfx);
                map_bitmaps[path].gfx = 0;
                map_bitmaps[path].isLoaded = false;

            }
        break;
        case FILE_TYPE_SND:
            {

                std::map<std::string , SND_resource>::iterator it = map_sounds.find(path);
                if(it==map_sounds.end()) return;

                if(map_sounds[path].is_aquired) al_destroy_sample(map_sounds[path].snd);
                map_sounds[path].snd = 0;
                map_sounds[path].is_aquired = false;

            }
        break;
        case FILE_TYPE_FNT:
            {

                std::map<std::string , FNT_resource>::iterator it = map_fonts.find(path);
                if(it==map_fonts.end()) return;

                if(map_bitmaps[path].isInitialized) al_destroy_bitmap(map_bitmaps[path].gfx);
                map_bitmaps[path].gfx = 0;
                map_bitmaps[path].isLoaded = false;
                map_bitmaps[path].isInitialized = false;
                map_bitmaps[path].w = 0;
                map_bitmaps[path].h = 0;

            }
        break;
    }

}
