#include <iostream>
#include "transition.h"

Transition::Transition(int w, int h) {

    display_width  = 640;
    display_height = 480;

    transfering = false;

    a = 0;
    b = 0;

    frame_count = 0;
    frame_delay = 0;

    alpha = 0;

    type = TRANS_TYPE_RANDOM;
}

ALLEGRO_BITMAP * Transition::get() {

    return b;

}

void Transition::setup(ALLEGRO_BITMAP *bmp, int frames, TRANS_TYPE typ) {

    if(frames<=0) {
        if(transfering) cleanup();
        return ;
    }

    frame_count = 0;
    frame_delay = frames;

    type = typ;

    a = al_create_bitmap(display_width, display_height);
    b = al_create_bitmap(display_width, display_height);

    ALLEGRO_BITMAP * backbuf_tmp = al_get_target_bitmap();
    al_set_target_bitmap(a);
    al_draw_bitmap(bmp, 0,0, 0);
    al_set_target_bitmap(backbuf_tmp);

    transfering = true;

}

void Transition::cleanup() {

    al_destroy_bitmap(a);
    al_destroy_bitmap(b);
    a = 0;
    b = 0;

    frame_count = 0;
    frame_delay = 0;
    transfering = false;

}

void Transition::transfer(ALLEGRO_BITMAP *bmp) {

    frame_count++;
    if(frame_count>=frame_delay) {
        cleanup();
    } else {

        // Preparations
        alpha = frame_count*255/frame_delay;
        if(alpha<0) alpha=0;
        if(alpha>255) alpha=255;

        // Begin

        ALLEGRO_BITMAP * backbuf_tmp = al_get_target_bitmap();
        al_set_target_bitmap(b);

        al_draw_bitmap(a, 0,0, 0);

        for( int i=0; i<display_height; i++ ) {
            int alp = alpha-i*255/display_height;
            if(alp<=0) continue;
            if(alp>=255) al_draw_bitmap_region(bmp, 0,i,display_width,1, 0,i, 0);
            else al_draw_tinted_bitmap_region(bmp, al_map_rgba(255,255,255,alp), 0,i,display_width,1, 0,i, 0);
        }


        al_set_target_bitmap(backbuf_tmp);

    }

}

bool Transition::isTransfering() {

    return transfering;

}
